#include "GReATConfigMeta.h"
#include "GreatConfig.h"
#include<boost/tokenizer.hpp>

int tokenize_csv_lines( const std::string& line, std::vector< std::string>& tokens)
{
	tokens.clear();
	int n_toks= 0;
	boost::tokenizer< boost::escaped_list_separator< char> > 
		tok( 
			line.begin(), 
			line.end(), 
			boost::escaped_list_separator< char>( 0, ',', 0)
			);
	for( boost::tokenizer< boost::escaped_list_separator< char> >::iterator beg=tok.begin(); beg!=tok.end(); ++beg){
		tokens.push_back( *beg);
		++n_toks;
	}
	return n_toks;
}


std::string GReATConfigMeta::getFullPath( const std::string& currWD, const std::string& path)
{
	if ( path.empty())
		return "";
	// 'path' is absolute if it begins with '<drive letter>:/' -OR- '/'
	if ( ( path.size() >= 2) && ( ( path[ 0]=='\\') || ( path[ 1]==':') ))
	{
		// absolute
		return path;
	}
	else
	{
		// relative
		if ( currWD.empty())
			return path;
		else
			if ( currWD[ currWD.size()-1] == '\\')
				return currWD+ path;
			else
				return currWD+ "\\"+ path;
	}
}

// bit mask for MetaCodeGenModes
// bits are numbered as 76543210
bool GReATConfigMeta::isLibraryToGenerate( MetaCodeGenModes mcgm)
{
	// bit 4 must be set
	unsigned char mode= mcgm;
	return (mode & 0x10) == 0x10;
}
	
bool GReATConfigMeta::isLibraryDebug( MetaCodeGenModes mcgm)
{
	// bit 4 must be set, bit 0 must be clear
	unsigned char mode= mcgm;
	return  (mode & 0x11)== 0x10;
}

bool GReATConfigMeta::isLibraryRelease( MetaCodeGenModes mcgm)
{
	// bit 4 must be set, bit 0 must be set
	unsigned char mode= mcgm;
	return (mode & 0x11) == 0x11;
}

bool GReATConfigMeta::isLibraryStatic( MetaCodeGenModes mcgm)
{
	// bit 4 must be set, bit 3 must be clear
	unsigned char mode= mcgm;
	return (mode & 0x18) == 0x10;
}

bool GReATConfigMeta::isLibraryDynamic( MetaCodeGenModes mcgm)
{
	// bit 4 must be set, bit 3 must be set
	unsigned char mode= mcgm;
	return (mode & 0x18) == 0x18;
}

void GReATConfigMeta::getRootFolder( GR::RootFolder& rootFolder, const std::string& cfgFileDir)
{
	getTransformationDN( cfgFileDir);	// opens _sdnTransformation
	rootFolder= GR::RootFolder::Cast( _sdnTransformation.GetRootObject());
}

Udm::DataNetwork& GReATConfigMeta::getTransformationDN( const std::string& cfgFileDir)
{
	if ( false== _sdnTransformation.isOpen())
	{
		const std::string& transFile= getTransformationFileName();
		std::string fullPathTransFile= getFullPath( cfgFileDir, transFile);
		_sdnTransformation.OpenExisting( fullPathTransFile, "GR.xsd", Udm::CHANGES_LOST_DEFAULT);
	}
	return _sdnTransformation;
}
void GReATConfigMeta::getGeneralBackend( std::map< std::string, const Udm::DataNetwork*>& dns, const std::string& cfgFileDir)
{
	const std::string& projectFile= _metaInfo.UdmProjectFile();
	std::string fullPathProjectFile= getFullPath( cfgFileDir, projectFile);
	// check, if the extension is .xml, open differently..
	if ( isUdmProjectFile())
	{
		if ( false== _prj.IsOpen())
			_prj.OpenExisting( fullPathProjectFile);
		std::vector< Udm::DataNetwork*> prDns= _prj.GetDataNetworks();
		for( std::vector< Udm::DataNetwork*>::const_iterator i= prDns.begin(); i!= prDns.end(); ++i)
		{
			dns[ Uml::Diagram::Cast( (*i)->GetRootObject()).name()]= *i;
		}
	}
	else
	{
		if ( false== _dn.isOpen())
			_dn.OpenExisting( fullPathProjectFile);
		dns[ Uml::Diagram::Cast( _dn.GetRootObject()).name()]= &_dn;
	}
}

static std::string getExtension( const std::string& filename)
{
	std::string ext( filename);
	std::string::size_type findExt= filename.rfind( '.');
	if ( findExt != std::string::npos)
		ext.erase( ext.begin(), ext.begin()+ ++findExt);	// delete '.' too.
	return ext;
}

bool GReATConfigMeta::isUdmProjectFile()
{
	// if UdmProjectFile extension is '.udm', the it is udm project file, if else, then not.
	const std::string& projectFile= _metaInfo.UdmProjectFile();
	if ( std::string( "udm") == getExtension( projectFile))	// !!VIZA case insensitive compare would be better
		return true;
	return false;
}

std::string GReATConfigMeta::getUdmProjectFileName()
{
	return  _metaInfo.UdmProjectFile();
}

std::string GReATConfigMeta::getCrossLinkDiagramName()
{
	if ( !isUdmProjectFile())
		return "";
	Uml::Diagram xLinkDiagram;
	getCrossLinkDiagram( xLinkDiagram);
	return xLinkDiagram.name();
}

void GReATConfigMeta::getCrossLinkDiagram( Uml::Diagram& xLinkDiagram)
{
	xLinkDiagram= Uml::Diagram::Cast( getCrossLinkDN().GetRootObject());
}

Udm::DataNetwork& GReATConfigMeta::getCrossLinkDN()
{
	return _prj.GetCrossMetaNetwork();
}

// can't use const std::string& return type, because of UDM implementation sucks.
std::string GReATConfigMeta::getTransformationFileName()
{
	return _metaInfo.TransformationFile();
}

std::string GReATConfigMeta::incFiles() const
{
	const GReATConfig::UserCode& uc= _metaInfo.UserCode_child();
	if ( !uc)
		return "";
	return uc.IncFiles();
}

std::string GReATConfigMeta::libFiles() const
{
	const GReATConfig::UserCode& uc= _metaInfo.UserCode_child();
	if ( !uc)
		return "";
	return uc.LibFiles();
}

std::string GReATConfigMeta::incDirs() const
{
	const GReATConfig::UserCode& uc= _metaInfo.UserCode_child();
	if ( !uc)
		return "";
	return uc.IncDirs();
}

std::string GReATConfigMeta::libDirs() const
{
	const GReATConfig::UserCode& uc= _metaInfo.UserCode_child();
	if ( !uc)
		return "";
	return uc.LibDirs();
}

GReATConfigMeta::MetaCodeGenModes GReATConfigMeta::getMetaCodeGenMode() const
{
	const std::string STATIC_LIB_DEBUG= "Static Link Library Debug Configuration";
	const std::string STATIC_LIB_RELEASE= "Static Link Library Release Configuration";
	const std::string DYNAMIC_LIB_DEBUG= "Dynamic Link Library Debug Configuration";
	const std::string DYNAMIC_LIB_RELEASE= "Dynamic Link Library Release Configuration";
	const std::string NO_LIB= "No Library (Source code)";
	std::string mcgm= _metaInfo.MetaCodeGen();
	if ( STATIC_LIB_DEBUG== mcgm ) {
		return MCGM_STATIC_LIB_DEBUG;
	}
	if ( STATIC_LIB_RELEASE== mcgm ) {
		return MCGM_STATIC_LIB_RELEASE;
	}
	if ( DYNAMIC_LIB_DEBUG== mcgm ) {
		return MCGM_DYNAMIC_LIB_DEBUG;
	}
	if ( DYNAMIC_LIB_RELEASE== mcgm ) {
		return MCGM_DYNAMIC_LIB_RELEASE;
	}
	if ( NO_LIB== mcgm ) {
		return MCGM_NO_LIB;
	}
	return MCGM_INVALID;
}
	
