/* NAME:	GReATConfigRuntime
 * AUTHOR:	Attila Vizhanyo - viza@isis.vanderbilt.edu
 *
 * GReATConfig Runtime Info protection & convenience proxy
 */
#ifndef GREATCONFIGRUNTIME_H
#define GREATCONFIGRUNTIME_H

#include <set>
#include <map>
#include <string>

namespace GReATConfig
{
	class RuntimeInformation;
	class InputFile;
};

// Runtime Info proxy class
// Provides a protected & convience way to access various information 
// in the Runtime Info section of a GReATConfig model.
class GReATConfigRuntime
{
public:
	// ctor; user must pass a valid 'runtimeInfo', no validity check performed here.
	GReATConfigRuntime( const GReATConfig::RuntimeInformation& runtimeInfo) :
      _runtimeInfo( runtimeInfo)
		{}
	// dtor
	virtual ~GReATConfigRuntime() 
		{}
	// Returns with input file children of '_runtimeInfo'.
	virtual void getInputFiles( std::set< GReATConfig::InputFile>& inputFiles);
	// Returns with the file name attribute of the input file with file id 'fileID'.
	virtual bool getFileName( const std::string& fileID, std::string& fname);
	// Returns with the copy name attribute of the input file with file id 'fileID'.
	virtual bool getCopyName( const std::string& fileID, std::string& cname);

protected:
	// Returns with input file belonging to 'fileID'. Initially builds, then search '_inputFileMap'.
	bool getInputFile( const std::string& fileID, GReATConfig::InputFile& inputFile);

private:
	// Reference to runtime info section of GReATConfig model.
	const GReATConfig::RuntimeInformation& _runtimeInfo;
	// Keeps track of fileID - input file associations. 
	std::map< std::string, GReATConfig::InputFile> _inputFileMap;
};

#endif // GREATCONFIGRUNTIME_H
