#include "GReATUtils.h"
#include <stdlib.h> // for _ltoa()
#include <locale>

std::string upperBegin( const std::string& toConvert)
{
	if ( true == toConvert.empty())
		return std::string();
	char* upper= _strupr( strdup( toConvert.c_str()));
	// if toConvert.length == 1, then toConvert.substr copies 0 bytes.
	return std::string( upper).substr( 0, 1) + toConvert.substr( 1, toConvert.length()-1);
}

std::string lowerBegin( const std::string& toConvert)
{
	if ( true == toConvert.empty())
		return std::string();
	char* lower= _strlwr( strdup( toConvert.c_str()));
	// if toConvert.length == 1, then toConvert.substr copies 0 bytes.
	return std::string( lower).substr( 0, 1) + toConvert.substr( 1, toConvert.length()-1);
}

std::string longHexToString( long toConvert)
{
	char temp[ 100];
	_ltoa( toConvert, temp, 16);
	return std::string( temp);
}

std::string convertEscapeSequences( const std::string& toConvert)
{
	std::string escaped= "";
	for( unsigned int i=0; i< toConvert.length(); ++i)
	{
		std::string::const_reference currChar= toConvert[ i];
		if (currChar == static_cast< std::string::value_type>('\\'))
			escaped+= "\\\\";
		else if (currChar == static_cast< std::string::value_type>('\?'))
			escaped+= "\\\?";
		else if (currChar == static_cast< std::string::value_type>('\''))
			escaped+= "\\\'";
		else if (currChar == static_cast< std::string::value_type>('\"'))
			escaped+= "\\\"";
		else
			escaped+= currChar;
	}
	return escaped;
}

char* dropExtension( char* filename)
{
	char* ext= strrchr( filename, '.');
	if( NULL!= ext)
	{
		// 'ext' points to '.' -> 'ext+1' must be valid.
		if ( *(ext+1) != '\\')
			*ext= '\0';	// do not cut filenames with path but without extension, like: "..\x" , ".\x"
	}
	return filename;
}

std::string dropExtension( const std::string& filename)
{
	std::string nameNoExt( filename);
	std::string::size_type findExt= nameNoExt.rfind( '.');
	if ( findExt != std::string::npos)
		if( *(nameNoExt.begin()+ findExt+ 1) != '\\')
			nameNoExt.erase( nameNoExt.begin()+ findExt, nameNoExt.end());	// delete '.' too.
	return nameNoExt;
}

std::string getExtension( const std::string& filename)
{
	std::string ext( filename);
	std::string::size_type findExt= ext.rfind( '.');
	if ( findExt != std::string::npos)
		if( *(ext.begin()+ findExt+ 1) != '\\')
			ext.erase( ext.begin(), ext.begin()+ ++findExt);	// delete '.' too.
		else
			ext.clear();
	return ext;
}

std::string dropPath( const std::string& filepath)
{
	std::string filename( filepath);
	std::string::size_type findPath= filename.rfind( '\\');
	if ( findPath != std::string::npos)
		filename.erase( filename.begin(), filename.begin()+ ++findPath);	// delete '\' too.
	return filename;
}

std::string getPath( const std::string& filepath)
{
	return getPath( filepath, false);
}

std::string getPath( const std::string& filepath, bool deleteTerminatingBackSlash)
{
	std::string filename( filepath);
	std::string::size_type findPath= filename.rfind( '\\');
	if ( findPath != std::string::npos)
		filename.erase( filename.begin()+ ( deleteTerminatingBackSlash ? findPath : ++findPath), filename.end());	// do not delete terminating '\'.
	else
		filename.erase();
	return filename;
}

std::string convertToCPlusPlusName( const std::string& name)
{
	// empty name
	if ( name.empty())
		return "";
	// name must start with letter
	std::string converted;
	if ( false== isalpha( name[ 0]))
		converted= "pre";
	// name can contain alphanum characthers and '_'
	for( std::string::const_iterator i= name.begin(); i!= name.end(); ++i)
	{
		if ( isalnum( *i) || (*i== '_'))
			converted+= *i;
		else
			converted+= 'X';
	}
	return converted;

}


