#include "CINTCodeGatherer.h"

#include "UdmBase.h"
#include "UdmDom.h"

#include "UmlExt.h"

#include "GR.h"
#include <fstream>
#include <direct.h>
// #include "C:\MoBIESTransition\GReAT\test\Udm\CrossLinks.h"
// #include "C:\MoBIESTransition\GReAT\test\Udm\TEST.h"
// #include "C:\MoBIESTransition\GReAT\test\Udm\CrossLinks.cpp"
// #include "C:\MoBIESTransition\GReAT\test\Udm\TEST.cpp"
#include "GReATUtils.h"

CINTCodeGatherer::~CINTCodeGatherer()
{
  closeDll();
}

void CINTCodeGatherer::Visit_RootFolder(const RootFolder& rf)
{
	string cppfilename = filename + ".cpp";
	string deffilename = filename + ".def";
	string headerfilename= filename + ".h";

	cppfile.open(cppfilename.c_str());
	deffile.open(deffilename.c_str());
	headerfile.open(headerfilename.c_str());

	deffile << "EXPORTS\n";

	cppfile << "// source file " << filename << ".cpp\n\n";

	cppfile << "#include <stdio.h>\n";
  cppfile << "#include <fstream>\n";
	cppfile << "#include \"UdmBase.h\"\n";
	cppfile << "#include \"UdmGme.h\"\n";
	
	cppfile << "#include \"UdmDom.h\"\n";
	cppfile << "#include \"UdmUtil.h\"\n";
	cppfile << "#include \"Uml.h\"\n";
	cppfile << "#include \"UmlExt.h\"\n";

	cppfile << "#include \""+ headerfilename + "\"\n";
  cppfile << "#include \"cint_string.h\"\n\n";

//generate user includes
  StrVec::const_iterator cit = m_cfg.m_incFiles.begin();
  for(; cit != m_cfg.m_incFiles.end(); ++cit)
  {
//    cppfile << "#include \""<< *cit << "\"" << std::endl;
	headerfile << "#include \""<< *cit << "\"" << std::endl;
  }

//generate meta includes
  StrUmlDiagramMap::const_iterator it = m_cfg.m_metas.begin();
  for(;it != m_cfg.m_metas.end(); ++it)
  {
//    cppfile << "#include \""<< m_cfg.m_udmDirName;
  //  cppfile << "\\" << it->first << ".h\"\n";
	headerfile << "#include \""<< m_cfg.m_udmDirName;
	headerfile << "\\" << it->first << ".h\"\n";
  }

  it = m_cfg.m_metas.begin();
  for(;it != m_cfg.m_metas.end(); ++it)
  {
    const std::string& mn = it->first;
  //init function
    deffile << "gz__init_" << mn << "\n";

    cppfile << "bool gz__init_" << mn << "(Uml::Diagram& dgr)\n{\n";
    cppfile << "\ntry\n{\n"; 
    cppfile << "\t" << mn << "::Initialize(dgr);\n"; 
    cppfile << "}\ncatch (const udm_exception& e)\n{\n";
    cppfile << "\tstd::cout<< std::string(e.what());\n";
    cppfile << "\treturn true;\n";
    cppfile << "}\n";
    cppfile << "\treturn false;\n}\n";

	headerfile << "extern bool gz__init_" << mn << "(Uml::Diagram& dgr);\n";
  }

	vector<RewritingRule> rules = rf.RewritingRule_children();
	for(vector<RewritingRule>::iterator itv =rules.begin(); itv!=rules.end(); ++itv)
	{	
    RewritingRule rule = *itv;
		rule.Accept(*this);
	}
		
	cppfile.close();
	deffile.close();
	headerfile.close();
}
 
void CINTCodeGatherer::Visit_RewritingRule(const RewritingRule& rule)
{
	path.push_back((string)rule.name());
	const string &rtype = rule.ruleType();
	if(rtype == "RULE" || rtype == "CASE")
	{	MakeGuardFunction(rule);
		MakeAttributeMappingFunction(rule);
	}
    	
	const vector<RuleBase> &subrules = rule.RuleBase_kind_children();
	for(vector<RuleBase>::const_iterator i=subrules .begin(); i!=subrules .end(); i++)
	{	const RuleBase &rb = *i;
		const string &srtype = rb.type().name();
		if( srtype != "RewritingRule" )
			continue;

		RewritingRule subrule = RewritingRule::Cast(rb);
		subrule.Accept(*this);
	}
	path.pop_back();
}
//============================================
bool CINTCodeGatherer::MakeGuardFunction(const RewritingRule& rule)
{	
	const string &guard = rule.guard();
	if(guard == "")
		return false;

	CreateSignature(rule, "Guard", "bool", true);
	cppfile << "\tbool Gz_guard = true; \n\tbool *guardVal= &Gz_guard;\n\t{\n\t\t";
	cppfile << rule.guard() << "\n\t}\n\treturn Gz_guard;\n";
	createBlockEnd(true);
  return true;
}
//============================================
bool CINTCodeGatherer::MakeAttributeMappingFunction(const RewritingRule& rule)
{
	const vector<Code> ams = rule.Code_kind_children();
	if(ams.size()==0)
		return false;

	CreateSignature(rule, "AM", "void", false);
	for(vector<Code>::const_iterator i=ams.begin(); i!=ams.end(); i++)
	{
		const Code am = *i;
		cppfile << "\n\t{\n\t\t" << am.code() << "\n\t}\n";
	}
	createBlockEnd(false);
  return true;
}
//============================================
void CINTCodeGatherer::createBlockEnd(bool guard)
{
   cppfile << "}\ncatch (const udm_exception& e)\n{\n";
   cppfile << "\tstd::cout<< std::string(e.what());\n";
   cppfile << "\terr = true;\n";
   if (guard)
     cppfile << "\treturn false;\n";

  cppfile << "}\n}\n";
}
//============================================
void CINTCodeGatherer::CreateSignature(const RewritingRule& rule, char *sigvar, char *ret, bool onlyBoundObjects)
{	const __int64 &o_id = rule.ruleId();
	
	vector<string>::iterator si=path.begin();
  cppfile << "//======================================\n";
	cppfile << "//Path : " << *si;
	for(si++;si!=path.end();si++)
	{	cppfile << "->" << *si;
	}
	cppfile << "\n";
	cppfile << ret << " " << sigvar << "_" << convertToCPlusPlusName( rule.name()) << "_"<< o_id << "( const vector<Udm::Object> &objs, bool& err ) \n{\n" ;
	headerfile << "extern " << ret << " " << sigvar << "_" << convertToCPlusPlusName( rule.name()) << "_"<< o_id << "( const vector<Udm::Object> &objs, bool& err );\n" ;
  cppfile << "\ntry\n{\n";
	cppfile << "\tvector<Udm::Object>::const_iterator obj_iter = objs.begin();\n";
	deffile << sigvar << "_" << convertToCPlusPlusName( rule.name()) << "_"<< o_id << "\n";

	set<string> names;
	typedef NameCompareFunctor< GR::PatternObject> PatternObjectCompareFunc;
	typedef std::set< GR::PatternObject, PatternObjectCompareFunc> PattObjSorted_t;
	PattObjSorted_t patobjs = rule.PatternObject_kind_children_sorted( PatternObjectCompareFunc());
	PattObjSorted_t::const_iterator i;
	for(i=patobjs.begin();i!=patobjs.end();i++)
	{	const PatternObject &patobj = *i;
		string name = patobj.name();

		// only bound objects support added by VIZA 08/25/04
		const std::string action= patobj.action();
		if ( onlyBoundObjects && ( "CreateNew" == action))
			continue;

		set<string>::iterator ni = names.find(name);
		if(ni!=names.end())
			continue;
		names.insert(name);
    std::string typeName = std::string(patobj.paradigmName()) + "::" + std::string(patobj.classType());

    cppfile << "\t" << typeName << " " << patobj.name() << " = ";
    cppfile << typeName << "::Cast(*obj_iter);\n";
//		cppfile << "\tUdm::Object " << patobj.name() << " = *obj_iter;\n";
		cppfile << "\tobj_iter++;\n";
	}

	cppfile << "\n";
}
                                          
//================================
void CINTCodeGatherer::generateMetaMakeInclude()
{
  std::string fname(m_cfg.m_udmDirName + "\\Gz__meta_include.mak");
  std::ofstream out(fname.c_str());
  out << "LINK32_OBJS = ";
  StrUmlDiagramMap::const_iterator it = m_cfg.m_metas.begin();
  for(;it != m_cfg.m_metas.end(); ++it)
  {
    out  << "\"$(INTDIR)\\" << it->first << ".obj\" ";
  }
  out << std::endl;
}
// //================================
// void CINTCodeGatherer::split(const std::string& str,
//                              StrVec& strVec,
//                              const std::string& delim)
// {
//   std::string::size_type of = 0;
//   while(true)
//   {
//     std::string::size_type f = str.find_first_of(delim, of);
// 
//     if (f != std::string::npos)
//     {
//       std::string tmp;
//       tmp = str.substr(of, f - of);
//       strVec.push_back(tmp);
//     }
//     else 
//     {
//       std::string tmp;
//       tmp = str.substr(of, str.size() - of);
//       strVec.push_back(tmp);
//       break;
//     }
// 
//     of  = f + delim.size();
//   }
// }
//================================
void CINTCodeGatherer::generateMainMakeInclude(const std::string& tmpDir,
                                               const std::string& metaBaseName)
{
  std::string fname(tmpDir + "\\tmp\\Gz__main_include.mak");
  
  std::ofstream out(fname.c_str());
  
  out << "USERS_LIB = ";
  std::string metaLibName(tmpDir + "\\tmp\\" + metaBaseName+ ".lib");
  out << "\"" << metaLibName << "\" ";


  StrVec::const_iterator it = m_cfg.m_libFiles.begin();
  for(; it != m_cfg.m_libFiles.end(); ++it)
  {
    out << "\"" << *it << "\" ";
  }

  out << std::endl;

  out << "USERS_INLCUDE_PATH = ";

  it = m_cfg.m_incDirs.begin();
  for(; it != m_cfg.m_incDirs.end(); ++it)
  {
    out << "/I \"" << *it << "\" ";
  }

  out << std::endl;

  out << "USERS_LIB_PATH = ";

  it = m_cfg.m_libDirs.begin();
  for(; it != m_cfg.m_libDirs.end(); ++it)
  {
    out << "/libpath:\"" << *it << "\" ";
  }

  out << std::endl;

}

#include <fstream>
typedef bool (*MAP_FN)(Uml::Diagram&);
//================================
void CINTCodeGatherer::generateDll(const UserCodeGenConfig& cfg) 
{
  m_cfg = cfg;
  const std::string& lastFName = cfg.m_depFileName;

  m_hDLL = 0; 
  HANDLE tFile;
	FILETIME ttCreate, ttLastAccess, ttLastWrite;
	tFile = CreateFile( lastFName.c_str(), GENERIC_READ, 0, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL );
	if(tFile != NULL)
	{	
    GetFileTime( tFile, &ttCreate, &ttLastAccess, &ttLastWrite );
		CloseHandle(tFile);
	}

	// Code to generate the cint code file and load it
			
  std::string::size_type end   = lastFName.find_last_of(".");
  std::string::size_type begin =  lastFName.find_last_of("\\");
  if (std::string::npos == begin)
    begin = 0;
  std::string fnamebase = lastFName.substr(begin + 1 , end - begin -1);

  long BUFSIZE = 500;
  TCHAR tchBuf[500]; 
    
  std::string greatPath;
  if (GetEnvironmentVariable("GREAT_PATH",tchBuf, BUFSIZE))
  {
     greatPath = std::string(tchBuf);
     filename = greatPath + "\\tmp\\" ;
    _mkdir(filename.c_str());
    
    filename += fnamebase; 
    
  }
  else 
    throw udm_exception("Error: Make process of Dll failed. Unable to find variable GREAT_PATH");

  std::string dllName;
  std::string libName;
  
  if (m_cfg.m_debug)
  {
    dllName = fnamebase  + "_D_dynamic";
    libName = fnamebase  + "_D_static";
    m_dllFullName = filename + "_D_dynamic.dll";
  }
  else
  {
    dllName = fnamebase  + "_dynamic";
    libName = fnamebase  + "_static";
    m_dllFullName = filename + "_dynamic.dll";
  }

  
	bool genDll = false;

	HANDLE dFile;
	FILETIME dtCreate, dtLastAccess, dtLastWrite;
	dFile = CreateFile( m_dllFullName.c_str(), GENERIC_READ, 0, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL );
  if((long) dFile != -1)
	{	
    GetFileTime( dFile, &dtCreate, &dtLastAccess, &dtLastWrite );
		if(CompareFileTime(&ttLastWrite, &dtLastWrite) == 1)
		{	
      genDll = true;
		}
		CloseHandle(dFile);
	}
	else
	{
    genDll = true; 
	}
	
	if(genDll)
	{
    
    generateMetaMakeInclude();

    std::string metaBaseName = fnamebase + "_meta";

    if(m_cfg.m_debug)
      metaBaseName += "_D"; 


    generateMainMakeInclude(greatPath, metaBaseName);

		GR::RootFolder rf = GR::RootFolder::Cast(cfg.m_transf->GetRootObject());
		rf.Accept(*this);
    
   
		cout << endl << "Creating and compiling the dll" << endl;
    std::string compileMode("CFG=Release");

    if (m_cfg.m_debug)
 		  compileMode = "CFG=Debug";

    _getcwd( tchBuf, BUFSIZE);
    std::string cwd(tchBuf);

    string makeCall = "call \"%GREAT_PATH%\\etc\\Gz__cint_make.bat\"  ";
    
    makeCall += "\""+ compileMode + "\"  ";
    makeCall += "\"BASENAME =" + fnamebase + "\"  ";
    makeCall += "\"DLL_NAME =" + dllName + "\"  ";
    makeCall +=  "\""+ cwd + "\"  ";
    makeCall += "\"" + cfg.m_udmDirName + "\" ";
    makeCall += "\"META_BASENAME=" + metaBaseName + "\" ";
    makeCall += "\"LIB_NAME =" + libName + "\"";

		int sysret = system(makeCall.c_str());
		if(sysret == 1)
		{	
			throw udm_exception("Error: Make process of Dll failed. Unable to find vcvars32.bat. Please add <visual studio>/vc98/bin to system path.");
		}
		else if(sysret == 2)
		{	
			throw udm_exception("Error: Make process of Dll failed. Please run command line version of Engine for details.");
		}
	}		
}
//================================
HINSTANCE  CINTCodeGatherer::loadDll()
{

	m_hDLL = LoadLibrary(m_dllFullName.c_str());

  if (m_hDLL == 0)
    throw udm_exception("Dll handler is null");

  StrUmlDiagramMap::iterator it = m_cfg.m_metas.begin();
  for(;it != m_cfg.m_metas.end(); ++it)
  {
    const std::string& mn = it->first;
  
    std::string fn = "gz__init_" + mn ;

    MAP_FN lpMapFn = (MAP_FN)GetProcAddress(m_hDLL, fn.c_str());
    
    bool err = lpMapFn(it->second);
    if (err)
      throw udm_exception("Dll initialization failed");
  }
  return m_hDLL; 
}
//================================
void CINTCodeGatherer::closeDll()
{
  FreeLibrary(m_hDLL);
}