/*
Copyright (c) Vanderbilt University, 2000-2001
ALL RIGHTS RESERVED
Vanderbilt University disclaims all warranties with regard to this
software, including all implied warranties of merchantability
and fitness.  In no event shall Vanderbilt University be liable for
any special, indirect or consequential damages or any damages
whatsoever resulting from loss of use, data or profits, whether
in an action of contract, negligence or other tortious action,
arising out of or in connection with the use or performance of
this software.
*/


// UdmApp.cpp: implementation of the CUdmApp class.
// This file was automatically generated as UdmApp.cpp
// by UDM Interpreter Wizard on Monday, May 13, 2002 13:45:42

// Tihamr Levendovszky 05-13-02

#include "stdafx.h"
#include <afxdlgs.h> // For CFileDialog
#include "resource.h"
//#include "OldNewDialog.h"
#include <windows.h>	// for cursors
//#include <iostream.h>
#include <atlbase.h>
#include <comdef.h>

#include <time.h>
#include "UdmStatic.h"
#include "UdmDom.h"
#include "UdmGME.h"
#include "UmlExt.h"
#include "UdmUtil.h"

#include "UdmApp.h"
#include "UdmConfig.h"

#include "UMLModelTransformer.h"

#include "ConfigGenerator.h"
#include "GReATConfigMeta.h"


extern void dummy(void); // Dummy function for UDM meta initialization

/*********************************************************************************/
/* Initialization function. The framework calls it before preparing the backend. */
/* Initialize here the settings in the config static object.					 */
/* Return 0 if successful.														 */
/*********************************************************************************/
int CUdmApp::Initialize()
{
	// TODO: Your initialization code comes here...
	return 0;
}



/* 
Remarks to CUdmApp::UdmMain(...):
0.	The p_backend points to an already open backend, and the framework 
	closes it automatically. DO NOT OPEN OR CLOSE IT!
	To commit changes use p_backend->CommitEditSequence().
	To abort changes use p_backend->AbortEditSequence().
	To save changes to a different file use p_backend->SaveAs() or p_backend->CloseAs().

1.	Focus is the currently open model.

2.	The possible values for param (from GME DecoratorLib.h component_startmode_enum):
	GME_MAIN_START			=   0,
	GME_BROWSER_START		=   1,
	GME_CONTEXT_START		=   2,
	GME_EMBEDDED_START		=   3,
	GME_MENU_START			=  16,
	GME_BGCONTEXT_START		=  18,
	GME_ICON_START			=  32,
	METAMODEL_CHECK_SYNTAX	= 101

 3. The framework catches all the exceptions and reports the error in a message box,
	clean up and close the transactions aborting the changes. You can override this 
	behavior by catching udm_exception. Use udm_exception::what() to form an error 
	message.
*/

/***********************************************/
/* Main entry point for Udm-based Interpreter  */
/***********************************************/

void CUdmApp::UdmMain(
					 Udm::DataNetwork* p_backend,		// Backend pointer(already open!)
					 Udm::Object focusObject,			// Focus object
					 set<Udm::Object> selectedObjects,	// Selected objects
					 long param)						// Parameters
{	
	// initialize CG error and warning global counters
	g_cntWrn= 0;
	g_cntErr= 0;
 	try
 	{
		::SetCursor( ::LoadCursor( 0, IDC_WAIT));
    UMLModelTransformer::RootFolder rf = UMLModelTransformer::RootFolder::Cast(p_backend->GetRootObject());
		UMLModelTransformer::Configuration cnf;
		vector<UMLModelTransformer::Configuration> confs;

		// Get the configuration model
		if(focusObject != Udm::null && (string)focusObject.type().name() == "Configuration")
		{	cnf = UMLModelTransformer::Configuration::Cast(focusObject);
		}
		else
		{	vector<UMLModelTransformer::Configurations> cfolds = rf.Configurations_children();
			for(vector<UMLModelTransformer::Configurations>::iterator i=cfolds.begin(); i!=cfolds.end();i++)
			{	UMLModelTransformer::Configurations cfold = *i;
				vector<UMLModelTransformer::Configuration> cfgs = cfold.Configuration_kind_children();
				confs.insert(confs.end(), cfgs.begin(), cfgs.end());
			}
			if(confs.size() == 0)
			{	throw udm_exception("Error: Couldnt find any configuration file");
			}
			if(confs.size() > 1)
			{	throw udm_exception("Error: More than one configuration found. Please make the desired one the focused object and then press interepret");
			}
			cnf = *confs.begin();
		}
    
		//
		const std::string& configFileName= cnf.ConfigFile();
		const std::string& CodeGenFileName= cnf.CodeGenFileName();

		string ConfigDir;
		int ne = configFileName.find_last_of('\\');
		ConfigDir = configFileName.substr(0,ne+1);
		
		string CodeGenDir;
		ne = CodeGenFileName.find_last_of('\\');
		CodeGenDir = CodeGenFileName.substr(0,ne+1);
		
		CodeGenDir = ConfigDir + CodeGenDir;
		
		if(CreateDirectory(CodeGenDir.c_str(), NULL)==0)
		{	int le = GetLastError();
			if(le != 183)
			{	LPVOID lpMsgBuf;
				FormatMessage( FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS,
								NULL, le, 0, (LPTSTR) &lpMsgBuf, 0, NULL );
				string msg = "CG Error: Unable to create directory " + CodeGenDir + "\n";
				msg += (LPCSTR)lpMsgBuf;
				throw udm_exception(msg.c_str());
			}
		}

		const std::string defFileNameBase= "output";
		const std::string& CodeGenFileNameBase= CodeGenFileName.empty() ? defFileNameBase : CodeGenFileName;
		ConfigGenerator cfgGen( configFileName);
		std::string currWD= getPath( configFileName);
		createLogFile( currWD);
		PE::resetPE_UID();
		// generate translator files
		cfgGen.generate();
		cfgGen.save( GReATConfigMeta::getFullPath( currWD, dropExtension( CodeGenFileNameBase.c_str())));
		// generate main.cpp file, if possible
		bool isMainGenerated= true;
		try
		{
			cfgGen.generateMain();
			cfgGen.saveMain( 
				GReATConfigMeta::getFullPath( currWD, dropExtension( CodeGenFileNameBase)) + std::string( "_main.cpp"), 
				GReATConfigMeta::getFullPath( currWD, dropExtension( CodeGenFileNameBase))
				);
		}
		catch ( CGEx&)
		{
			isMainGenerated= false;
		}
		closeLogFile();
		// 
		std::string successMsg= "The following files have been successfully generated: \n\n"+ 
			currWD+ "\n\t"+
			(isMainGenerated ? dropExtension( CodeGenFileNameBase)+ std::string( "_main.cpp\n\t") : "") +
			dropExtension( CodeGenFileNameBase)+ std::string( ".h\n\t")+
			dropExtension( CodeGenFileNameBase)+ std::string( ".cpp\n\n")+
			num_to_string( g_cntErr) + " error(s), " + num_to_string( g_cntWrn) + " warning(s).\n"+
			"See TranslatorLog.txt for details.";
		AfxMessageBox( successMsg.c_str(), MB_ICONINFORMATION);
	}
	catch(udm_exception &e)
	{
		closeLogFile();
		::SetCursor( ::LoadCursor( 0, IDC_ARROW));
		AfxMessageBox(e.what());
	    throw e; // letthe master interpreter know that something went wrong ... /kalmar
	}
	catch (CGEx& e)
	{
		closeLogFile();
		::SetCursor( ::LoadCursor( 0, IDC_ARROW));
		AfxMessageBox( e.what().c_str());
	    throw e; 
	}
	::SetCursor( ::LoadCursor( 0, IDC_ARROW));
}



#ifdef _DEBUG
/*****************************************************/
/* Debug time helper function. If the object has an  */
/* attribute called "name", this function retreives  */
/* it to help you to find it in the model during the */
/* application development.	Usualy every GME Object	 */
/* has a "name" attribute. If an object hapens not	 */
/* to have it,function retreives <no name specified>.*/
/*****************************************************/
string CUdmApp::ExtractName(Udm::Object ob)
{
	Uml::Class cls= ob.type();				
	set<Uml::Attribute> attrs=cls.attributes();		
	
	// Adding parent attributes
	set<Uml::Attribute> aattrs=Uml::AncestorAttributes(cls);
	attrs.insert(aattrs.begin(),aattrs.end());

	for(set<Uml::Attribute>::iterator ai = attrs.begin();ai != attrs.end(); ai++) 
	{
		if(string(ai->type())=="String")
		{
			string str=ai->name();
			if(str=="name")
			{
				string value=ob.getStringAttr(*ai);
				if(value.empty())value="<empty string>";
				return value;
			}			
		}				
	}	
	return string("<no name specified>");
}
#endif
