/* NAME:	CINT Compiler
 * AUTHOR:	Attila Vizhanyo - viza@isis-server.isis.vanderbilt.edu
 *
 * Compiles CINT expression strings into generated code (such as guards, codes)
 */
#ifndef __CINTCOMPILER
#define	__CINTCOMPILER

#ifdef _MSC_VER
#pragma warning (disable : 4786)
#pragma warning (disable : 4503) 
#endif

#include "RuleTypes.h"
#include "PE.h"

#include "ICIntCompiler.h"

// This class is not designed to be a base class. 
// Either don't inherit from it, or add virtual destructor.
class CIntCompiler : virtual public ICIntCompiler
{
public:
	// ctor, see parameter desc. at the member variables declaration part.
	CIntCompiler() : _rule( 0), _boundObjects( 0), _addGuardFuncDefTo( 0), _currCode( 0)
	{}
	// user must call init() before use.
	void init( const GRSimpleRule& rule, 
		const std::map< GR::ObjectWrapper, std::string>& boundObjects,
		Class& addGuardFuncDefTo
		) 
	{
		_rule= &rule;
		_boundObjects= &boundObjects;
		_addGuardFuncDefTo= &addGuardFuncDefTo;
		_currCode= NULL; 
		_owNameMap.clear();
	}
	// Returns true, if '_rule' has guard defined.
	bool isGuard() const;
	// Returns true, if '_rule' has any attribute mapping code elements.
	bool isCode() const;
	// Create code into 'code', which executes guards.
	// Returns with the guard variable name.
	const std::string createGuard( PEComposite& code);
	// Create code into 'code', which executes attribute mapping code elements.
	void createCode( PEComposite& code);

protected:
	// Create guard declaration/ initialization code into '_currCode'.
	// Returns with the guard variable name.
	const std::string createGuardInit();
	// Create code into '_currCode', which initialize UdmPseudoObjects with object wrappers defined in '_rule'.
	void passObjects();
	//
	void passParameters( Function& guardFuncDef, FunctionCall& guardFuncCall);
	//
	// If 'filterBoundObjects' set to true, only object wrappers with action 'Bound' will be generated.
	void buildOWNameMap( bool onlyBoundObjects);

private:
	// Rule of which CInt expression strings are being compiled.
	const GRSimpleRule* _rule;
	// Maps object wrappers in '_rule' to their generated variable names.
	const std::map< GR::ObjectWrapper, std::string>* _boundObjects;
	//
	Class* _addGuardFuncDefTo;
	// pointer to program element, which is the current target of cint compiler code generation.
	PEComposite* _currCode;

protected:
	// Maps ow name to ow object & unique flag.
	typedef std::map< std::string, std::pair< GR::ObjectWrapper, bool> > OWNameMap_t;
	OWNameMap_t _owNameMap;	
};

#endif //__CINTCOMPILER