#include "CINTLibraryCall.h"
#include "GReATUtils.h"
#include "PEStringTable.h"

const std::string AM_PREFIX= "AM_";
const std::string GUARD_PREFIX= "Guard_";

bool CIntLibraryCall::isGuard() const
{
	return CIntCompiler::isGuard();
}

bool CIntLibraryCall::isCode() const
{
	return CIntCompiler::isCode();
}

const std::string CIntLibraryCall::createGuard( PEComposite& code)
{
	const std::string GUARD_VAR_NAME= "Gz_guard";

	buildOWNameMap( true);
	std::string objVecName= createObjectVector( code);
	fillObjectVector( objVecName, code);
	//
	std::string funcName= GUARD_PREFIX+ convertToCPlusPlusName( ( std::string) _rule->name())+ "_" + num_to_string( _rule->ruleId());
	std::string errName= createErrVariable( code);
	code.add( *new Expression( BOL+ SEP+ GUARD_VAR_NAME+ EQ+ SEP+ funcName+ "(" + SEP+ objVecName+ ","+ SEP+ errName+ ")"));
	return GUARD_VAR_NAME;
}

void CIntLibraryCall::createCode( PEComposite& code)
{
	buildOWNameMap( false);
	std::string objVecName= createObjectVector( code);
	fillObjectVector( objVecName, code);
	//
	std::string funcName= AM_PREFIX+ convertToCPlusPlusName( ( std::string) _rule->name())+ "_" + num_to_string( _rule->ruleId());
	std::string errName= createErrVariable( code);
	code.add( *new Expression( funcName+ "(" + SEP+ objVecName+ ","+ SEP+ errName+ ")"));

}

std::string CIntLibraryCall::createObjectVector( PEComposite& addTo) const
{
	Definition* vecDef= new Definition;
	vecDef->_type= "std::vector< Udm::Object>";
	vecDef->_variable= "objs";
	addTo.add( *vecDef);
	return vecDef->_variable;
}

void CIntLibraryCall::fillObjectVector( const std::string objVecName, PEComposite& addTo) const
{
	std::set< std::string> namesInsertedIntoVector;
	for( OWNameMap_t::const_iterator itNames= _owNameMap.begin(); itNames!= _owNameMap.end(); ++itNames)
	{
		const GR::ObjectWrapper& currOW= itNames->second.first;

		std::string name= currOW.name();
		std::set< std::string>::const_iterator itNames = namesInsertedIntoVector.find( name);
		if( namesInsertedIntoVector.end() != itNames)
			continue;
		namesInsertedIntoVector.insert( name);

		if ( currOW.dataNetworkID() == -1)	// pseudo class
			continue;

		std::map< GR::ObjectWrapper, std::string>::const_iterator itBoundObject= _boundObjects->find( currOW);
		if ( _boundObjects->end()== itBoundObject) {
			throw CGEx("FATAL: CG internal error. Pattern object: "+ ( std::string) currOW.name()+ " not bound in CInt compiler.");
		}
		const std::string& generatedName= (*itBoundObject).second;
		addTo.add( *new Expression( objVecName+ ".push_back(" + SEP+ generatedName+ ")"));
	}
}

std::string CIntLibraryCall::createErrVariable(  PEComposite& addTo) const
{
	Definition* errDef= new Definition;
	errDef->_type= BOL;
	errDef->_variable= "err";
	addTo.add( *errDef);
	return errDef->_variable;
}
