/* NAME:	General backend
 * AUTHOR:	Attila Vizhanyo - viza@isis-server.isis.vanderbilt.edu
 *
 * General backend maintains 'host meta name' - 'Udm data network' relations. 
 * (Implemented by mapping)
 */
#ifndef __GENERALBACKEND_H
#define __GENERALBACKEND_H

#ifdef _MSC_VER
#pragma warning (disable : 4786) 
#pragma warning (disable : 4503)
#endif

#include <algorithm>	// for 'for_each'
#include <map>
#include <Uml.h>
#include "GReATUtils.h"	// for 'DeleteMapValueFunctor'
#include "CGEx.h"

// This class is not designed to be a base class. 
// Either don't inherit from it, or declare destructor virtual.
class GeneralBackend
{
public:
	// ctor & dtor
	GeneralBackend( bool deleteBackendsWhenDie= false) : _delete( deleteBackendsWhenDie) {}
	~GeneralBackend()
	{
		if ( _delete)
			std::for_each( _backendMap.begin(), _backendMap.end(), DeleteMapValueFunctor());
	}
	// Register data network 'dn' with name 'name' into '_backendMap'
	void addBackend( const std::string& name, Udm::DataNetwork& dn);
	// Set cross link backend (this is optional).
	void setCrossLinkBackend( const std::string& name, Udm::DataNetwork& dn);
	// Retrieve data network with name 'name' from '_backendMap'. If not found, return NULL.
	const Udm::DataNetwork* getBackend( const std::string& name) const;
	// Get diagram of the data network with name 'name'. If not found, throws CGEx!
	const Uml::Diagram getDiagram( const std::string& name) const throw (CGEx);
	// Returns with the diagram of the cross link dn.
	const Uml::Diagram getCrossLinkDiagram() const throw (CGEx);
	// Get all dn-s. (Cross link dn, too.)
	typedef std::map< std::string, const Udm::DataNetwork*> BackendMap_t;
	const BackendMap_t& getAllBackends() const { return _backendMap;}
	// Set 'free-up dn-s on destruction' flag.
	void setDelete( bool deleteBackendsWhenDie= false) { _delete= deleteBackendsWhenDie;}
	// Copy 'dns' into this->_backendMap.
	GeneralBackend& operator=( const BackendMap_t& dns);

private:
	// Maps meta names to data network pointers.
	BackendMap_t _backendMap;
	// If '_delete' is set to true, then dtor will delete data networks in '_backendMap'.
	bool _delete;
	// Cross link diagram name.
	std::string _xlinkDiagName;
};

#endif //__GENERALBACKEND_H
