#ifndef CREATEDNCODEPRINTER_H
#define CREATEDNCODEPRINTER_H

#include <string>

#include "DNProjectCodePrinter.h"
#include "CopyToMemDnCodePrinter.h"
#include "SmartDNCodePrinter.h"

// Abstract Factory
class CreateDNCodePrinter
{
public:
	virtual DNCodePrinter* create( const std::string& mode) = 0;
};

// Creates and returns with the Udm project code printer relevant to 'mode'.
class CreateDNProjectCodePrinter : public CreateDNCodePrinter
{
public:
	virtual DNCodePrinter* create( const std::string& mode)
	{
		DNCodePrinter* dnCodePrinter= 0;
		if ( "r" == mode) {
			dnCodePrinter= new ReadDNProjectCodePrinter();
		}
		else if ( "w" == mode) {
			dnCodePrinter= new WriteDNProjectCodePrinter();
		}
		else if ( "rw" == mode) {
			dnCodePrinter= new ReadWriteDNProjectCodePrinter();
		}
		else if ( "rwc" == mode) {
			dnCodePrinter= new WriteCopyDNProjectCodePrinter();
		}
		return dnCodePrinter;
	}
};

class CreateCopyToMemDnCodePrinter : public CreateDNCodePrinter
{
public:
	virtual DNCodePrinter* create( const std::string& mode)
	{
		CopyToMemDNCodePrinter* dnCodePrinter= 0;
		if ( "r" == mode) {
			dnCodePrinter= new ReadCopyToMemDNCodePrinter();
		}
		else if ( "w" == mode) {
			dnCodePrinter= new WriteCopyToMemDNCodePrinter();
		}
		else if ( "rw" == mode) {
			dnCodePrinter= new ReadWriteCopyToMemDNCodePrinter();
		}
		else if ( "rwc" == mode) {
			dnCodePrinter= new WriteCopyCopyToMemDNCodePrinter();
		}
		return dnCodePrinter;
	}
};

// Creates and returns with the smart datanetwork code printer relevant to 'mode'.
class CreateSmartDNCodePrinter : public CreateDNCodePrinter
{
public:
	virtual DNCodePrinter* create( const std::string& mode)
	{
		DNCodePrinter* dnCodePrinter= 0;
		if ( "r" == mode)
		{
			dnCodePrinter= new SmartDNReadOnlyCodePrinter();
		}
		else if ( "w" == mode)
		{
			dnCodePrinter= new SmartDNWriteOnlyCodePrinter();
		}
		else if ( "rw" == mode)
		{
			dnCodePrinter= new SmartDNReadWriteCodePrinter();
		}
		else if ( "rwc" == mode)
		{
			dnCodePrinter= new SmartDNWriteCopyCodePrinter();
		}
		return dnCodePrinter;
	}
};


#endif //CREATEDNCODEPRINTER_H