#include "GRDebuggerConsol.h"
#include <iostream>
#include <string>
using namespace GRDI;

//=========================================
GRDebuggerConsol::GRDebuggerConsol() : 
  m_debuggerCtrl(0),
  m_init(false)
{
}
//=========================================

void GRDebuggerConsol::setIGRDebuggerCtrl(GRDI::IGRDebuggerCtrl* debuggerCtrl)
{
  m_debuggerCtrl = debuggerCtrl;
  m_init = true;
}
//=========================================
GRDI::IGRDebuggerView* GRDebuggerConsol::getIGRDebuggerView()
{
  return static_cast<GRDI::IGRDebuggerView*> (this);
}
//=========================================
void GRDebuggerConsol::operator()()
{
  run();
}
//=========================================
void GRDebuggerConsol::run()
{
#ifndef _DEBUG 
try
{
#endif

  while(!m_init);

  if (m_debuggerCtrl)
    m_debuggerCtrl->startCtrl();
  
  bool quit = false;
  std::string prev_cmd; 
  while(!quit)
  {
    std::string str;  
    //std::cin>>str;
    std::getline(std::cin, str, '\n');

    if (str.empty())
    {
      str = prev_cmd ;
    }
    if (m_debuggerCtrl)
      quit = m_debuggerCtrl->stepCtrl(str);
    else
    {
      if (str == "q")
        quit = true;
    }

    prev_cmd = str;
  }
  if (m_debuggerCtrl)
    m_debuggerCtrl->stopCtrl();
#ifndef _DEBUG 
}
catch(...)
{
 std::string str("exception !!!");
 std::cout << str << std::endl;
// lout.print(1,"GRDebugger",str);
}
#endif
}

//======IGRDebuggerView

void GRDebuggerConsol::print(const std::string& out)
{
  Lock lk(m_mutexPrint);
  std::cout << std::endl << out << std::endl << ">";
}
//=========================================
void GRDebuggerConsol::printLoc(const LocationID& locID,
                                const std::string& name, 
                                std::stringstream& str,
                                const LocationID& markLocID,
                                int breakpoint)
{
   if (markLocID == locID)
   {
     if (breakpoint)
      str << "@ ";
     else
      str << ">> ";
   }
   else
     str << "   ";
       
   str << locID << "    " << name << std::endl;
}

//=========================================
void GRDebuggerConsol::markFuncLocationImpl(  std::stringstream& str,
                             GRDI::IGRDebuggerView::FuncLocationMarkDescrVec& markLocs)
{
  GRDI::IGRDebuggerView::FuncLocationMarkDescrVec::const_iterator it = markLocs.begin();
  for(; it != markLocs.end(); ++it)
  {
    const GRDI::LocationID& markLocID = it->m_loc.m_locID;
    const int& mark = it->m_mark;
    const FunctionDescr& descr = it->m_descr;
    buildFunctionSourceConsolImpl(it->m_loc, descr, str, markLocID, mark);
  }
}
//=========================================
void GRDebuggerConsol::buildFunctionSourceImpl(
                                               const Location& loc,
                                               const FunctionDescr& descr, 
                                               std::stringstream& str)
{
  buildFunctionSourceConsolImpl(loc, descr, str);
}

//=========================================
void GRDebuggerConsol::buildFunctionSourceConsolImpl(
                                               const Location& loc,
                                               const FunctionDescr& descr, 
                                               std::stringstream& str,
                                               const LocationID& markLocID,
                                               int breakpoint)
{
  str << std::endl << "     function " << *(descr.m_func);

  if (descr.m_varTable.size())
  {
    str << " ( ";
    const VariableValueMap& vars = descr.m_varTable;
    VariableValueMap::const_iterator itVars = vars.begin();
    for(;itVars != vars.end(); ++itVars)
    {
      if (itVars != vars.begin()) 
       str << ", ";

      str << *(itVars->first);
    }
    str << " )";
  }
  if (descr.m_ref)
   str << "->" << *(descr.m_ref);

  str<< std::endl;

 if (descr.m_ref == GRDI::IFunctionPtr())
 {
  const LocationIDVec&  locations = descr.m_callLocs;
  const FunctionTable&  calls = descr.m_calls;    

  printLoc(locations.front(), "begin", str, markLocID, breakpoint);

  if (calls.size())
  {
    for(FunctionTable::size_type i = 0; i < calls.size(); ++i)
     {
       const LocationID& callLocID = locations[i + 1];
       std::stringstream name;
       name << *(descr.m_calls[i]);
       printLoc(callLocID , name.str(), str, markLocID, breakpoint);
     }
  }
  printLoc(locations.back(), "end", str, markLocID, breakpoint);
 }
}
//=========================================
void GRDebuggerConsol::buildSourceImplBegin(std::stringstream& str)
{
}

//=========================================
void GRDebuggerConsol::buildSourceImplEnd(std::stringstream& str)
{
  m_sourceTxt.clear();
  
  str << std::endl;

  m_sourceTxt = str.str();
}
//=========================================
void GRDebuggerConsol::printSource()
{
  print(m_sourceTxt);
}

//=========================================
void GRDebuggerConsol::printContextImpl(const ContextDescr& contextDescr, 
                                  std::ostream& str,
                                  bool exception)
{
  str << "------------------" << std::endl;
  str << "context id: "  << contextDescr.m_id << std::endl;
  str << "location: " <<  contextDescr.m_currLocationID << std::endl;
     

  str << "function: "  << *(contextDescr.m_func) << std::endl;

  if (!exception)
  {
    const VariableValueMap& valMap = contextDescr.m_varValMap;
    if (valMap.size())
    {
      str << "variables:" << std::endl;

      VariableValueMap::const_iterator it = valMap.begin(); 

      for (;it != valMap .end(); ++it)
      {
      
        str << "          ";
        str << *(it->first) << ": ";

        if (it->second) 
        {
          it->second->print(str, 20);
        }
        else
          str << "has no value";
        str << std::endl;
      }
    }
    else
      str << "no variables" << std::endl;
  }
  else
    str << "variables are not available" << std::endl;
}

//=========================================
void GRDebuggerConsol::printContextStackImplEnd( 
                                  std::ostream& str,
                                  bool exception)
{
}
//=========================================
void GRDebuggerConsol::printContextStackImplBegin(std::ostream& str, bool exception)
{
    std::string ss(((std::stringstream&)str).str());
    
    ss.insert(0, "-------------\nContext Stack\n");
    print(ss);
}
//=========================================
void GRDebuggerConsol::stopped()
{
}
//=========================================
void GRDebuggerConsol::exception()
{
}
//=========================================
void GRDebuggerConsol::finished()
{
}
//=========================================
void GRDebuggerConsol::loaded()
{
}
//=========================================
void GRDebuggerConsol::saved()
{
}

//=========================================
void GRDebuggerConsol::closed()
{
}

//=========================================
void GRDebuggerConsol::hitBreakpoint()
{
}
//=========================================
void GRDebuggerConsol::step()
{
}
//=========================================
void GRDebuggerConsol::printCurrentProgress(double prg)
{
  std::stringstream ss;
  ss << "Progress satatus: "<< prg << std::endl;
  print(ss.str());
}

void GRDebuggerConsol::progressReport(double)
{
}