#ifndef GReATDebugger_H
#define GReATDebugger_H


#include "GReATDebuggerVars.h"
#include "IGReAT.h"
#include "../GRLauncher/GRDebuggeeAPI.h"
#include "../GRDebugInterface/GRDebugTypes.h"




typedef std::map<GR::RuleBase, GRDI::FunctionDescr> RuleFunctionDescrMap;
typedef std::map<GR::ObjectWrapper, GRDI::IVariablePtr> ObjectWrapperIVariablePtrMap;



namespace GRDI
{
  class IGRDBackEnd;
}

class IGReATParameters;
class ICINTCodeGatherer;

class GReATDebugger :   public   GRDebuggeeAPI,
                        public   IGReAT::IDebugNotify,
                        public   IGReAT::ICtrlNotify
    
                        
{

protected:

  IGReAT*             m_great;
  ICINTCodeGatherer*  m_cintDll;
  GRDI::IGRDBackEnd*  m_back;


  bool                m_loaded;

  RuleFunctionDescrMap  m_sourceMap;

  ObjectWrapperIVariablePtrMap m_varMap;

  IGReATParameters*    m_grParams;

  bool                  m_up;
  bool                  m_init;
  

  std::string           m_paramFileName;
  std::string           m_workingDirectory;
  bool                  m_debugMode;
  bool                  m_quitNonDebug;
  GeneralBackendPtr     m_generalBackEnd;
  double                m_progress; 
protected:

  void buildSource(); 
  void clearSource();
  void runDebugMode();
  void runNonDebugMode();
  bool execute();

  void handleGRParamsException(const std::string& str);
  GRDI::IVariablePtr&  findVariable(const GR::ObjectWrapper& var);
  GRDI::IVariablePtr&  regVariable(const GR::ObjectWrapper& var);

  GRDI::FunctionDescr&  regFunction(const GR::RuleBase& rule, bool& exists);
  GRDI::FunctionDescr&  findFunction(const GR::RuleBase& rule);

  void buildFuncDependencies(const GR::RuleBase& rule);

  void addVariables(const GR::RewritingRule& rule,
                          GRDI::FunctionDescr& descr);
  void addMatchVarValue(GRDI::FunctionCallDescr& funcCallDescr,
                                   const MatchVector& match);


  GRDI::FunctionDescr&  addFunction(const GR::RuleBase& f);
  void addVarValue(GRDI::FunctionCallDescr& funcCallDescr,
                  const GR::RuleBase& rule,
                  const PassAlongPackets& arguments);

  void resetTest(const GRDI::FunctionDescr& descr,
                 const GR::RuleBase& rule);

  void setCase( const GR::RuleBase& rule, 
                const PassAlongPackets& arguments,
                GRDI::FunctionCallDescr& funcCallDescr);

  bool closeGRParams();
  bool loadGRParams(const std::string& pfn,
                    const std::string& wd);
  bool saveProgram();
  bool reloadProgram();
  bool closeProgram();
  void checkInterActions();
public:
  GReATDebugger();
  ~GReATDebugger(); 
  void operator()();

 //========== GRDebuggeeAPI
  void setWD(const std::string& wd);
  void load(const std::string& pfn);
  void run();
  void stop();
  void wait_up();
  //void close();
  void setGRDBackEnd(GRDI::IGRDBackEnd*  back);

//================== IGReAT::ICtrlNotify
  bool quit();
  void reportProgress(double);
//================== IGReAT::IDebugNotify
  void stepIn(const GR::RuleBase& rule, const PassAlongPackets& arguments);
  void step(const GR::RuleBase& caller, 
            const GR::RuleBase& callee,
            const PassAlongPackets& arguments);

  void stepOut(const GR::RuleBase& rule, 
               const PassAlongPackets& arguments,
               const MatchVector& = MatchVector());
};


#endif //GReATDebugger_H

