/*
 *	Copyright (c) Vanderbilt University, 2003-2004
 *	ALL RIGHTS RESERVED
 *
 *	Vanderbilt University disclaims all warranties with regard to this
 *	software, including all implied warranties of merchantability
 *	and fitness.  In no event shall Vanderbilt University be liable for
 *	any special, indirect or consequential damages or any damages
 *	whatsoever resulting from loss of use, data or profits, whether
 *	in an action of contract, negligence or other tortious action,
 *	arising out of or in connection with the use or performance of
 *	this software.
 *
 *  FILE NAME:	MetaGME2UML
 *	SYNOPSIS:	Class that converts a MetaGME metamodel to an equivalent UDM compatible UML class diagram
 *  AUTHOR:		Aditya Agrawal - aditya,agrawal@vanderbilt.edu
 *  CREATED:	03/19/03
 *  LAST UPDATE: 
 */

//#include "stdafx.h"
//#include <afxdlgs.h> // For CFileDialog

#include "MetaGME2UML.h"
#include "algorithm"
#include "cctype"

bool AskUserForConstraint(long);

void MetaGME2UML::transformMetaGME2UmlGME()
{	
	cout << endl;
	// Phase 1
	//MESSAGE_OUT("Phase 1");
	cout << "Phase 1 - Seting up the sheets .... " << endl;
	Phase1_SetUpSheets();
	// Phase 2
	//MESSAGE_OUT("Phase 2");
	cout << "Phase 2 - Creating Implementation & Interface Inheritance .... " << endl;
	Phase2_CreateImplementationInterfaceInheritances();
	// Phase 3
	//MESSAGE_OUT("Phase 3");
	cout << "Phase 3 - Creatig Classes .... " << endl;
	Phase3_CreateClasses();
	// Phase 4
	//MESSAGE_OUT("Phase 4");
	cout << "Phase 4 - Creatig ClassCopy .... " << endl;
	Phase4_CreateReferences();
	// Phase 5
	//MESSAGE_OUT("Phase 5");
	cout << "Phase 5 - Creatig Associations .... " << endl;
	Phase5_CreateConnections();
	// Phase 6
	//MESSAGE_OUT("Phase 6");
	cout << "Phase 6 - Creating and associating GME Object base .... " << endl;
	Phase6_CreateGMEObject();
	cout << "Done." << endl;
}

void MetaGME2UML::Phase1_SetUpSheets()
{	
	grf = MetaGME::RootFolder::Cast(metaGME.GetRootObject());
	InitializeSheets(urf);
	SetUpSheets(grf,urf);
}

void MetaGME2UML::SetUpSheets(MetaGME::RootFolder grf, Udm::Object urf)
{	
	vector<MetaGME::RootFolder> rfs = grf.RootFolder_kind_children();
	for(vector<MetaGME::RootFolder>::iterator ri=rfs.begin(); ri!=rfs.end();ri++)
	{	MetaGME::RootFolder sf = *ri;
		SetUpSheets(sf, urf);
	}
	
	vector<MetaGME::SheetFolder> sfs = grf.SheetFolder_kind_children();
	for(vector<MetaGME::SheetFolder>::iterator i=sfs.begin(); i!=sfs.end();i++)
	{	MetaGME::SheetFolder sf = *i;
		vector<MetaGME::ParadigmSheet> pss = sf.ParadigmSheet_kind_children();
		for(vector<MetaGME::ParadigmSheet>::iterator j=pss.begin(); j!=pss.end();j++)
		{	MetaGME::ParadigmSheet ps = *j;
			UmlGME::ClassDiagram cd = UmlGME::ClassDiagram::Create(urf);
			cd.name() = ps.name();
			cd.position() = ps.position();
			mapMetaGME2UmlGME_Paradigm[ps] = cd;

			SetupEquivalenceMaps(ps, cd);
		}
	}
	
	vector<MetaGME::ParadigmSheet> pss = grf.ParadigmSheet_kind_children();
	for(vector<MetaGME::ParadigmSheet>::iterator j=pss.begin(); j!=pss.end();j++)
	{	MetaGME::ParadigmSheet ps = *j;
		UmlGME::ClassDiagram cd = UmlGME::ClassDiagram::Create(urf);
		cd.name() = ps.name();
		cd.position() = ps.position();
		mapMetaGME2UmlGME_Paradigm[ps] = cd;		
		SetupEquivalenceMaps(ps, cd);
	}
}

void MetaGME2UML::SetupEquivalenceMaps(MetaGME::ParadigmSheet ps, UmlGME::ClassDiagram cd)
{	
	// Transfering Equivalence association
	{	vector<MetaGME::Equivalence> connects = ps.Equivalence_kind_children();
		for(vector<MetaGME::Equivalence>::iterator i = connects.begin(); i!=connects.end(); i++)
		{	MetaGME::Equivalence contain = *i;
			set<MetaGME::EquivLeft> basecs = contain.dstFCO();
			if(basecs.size() == 0)
			{	//Error
				string error = "Error: Equivalence object "+(string)contain.position()+ " in ParadigmSheet:" + (string)ps.name() + " is not left connected to an FCO";
				throw udm_exception(error);
			}
			if(basecs.size() > 1)
			{	//Error
				string error = "Error: Equivalence object "+(string)contain.position()+ " in ParadigmSheet:" + (string)ps.name() + " is left connected to more than one FCO";
				throw udm_exception(error);
			}
			MetaGME::EquivLeft basec = *(basecs.begin());
			MetaGME::BaseFCO base = basec.dstFCO_end();
			
			set<MetaGME::EquivRight> derivedcs = contain.srcFCO();
			if(derivedcs.size() == 0)
			{	//Error
				string error = "Error: Equivalence object "+(string)contain.position()+ " in ParadigmSheet:" + (string)ps.name() + " is not right connected to an FCO";
				throw udm_exception(error);
			}
			if(derivedcs.size() > 1)
			{	//Error
				string error = "Error: Equivalence object "+(string)contain.position()+ " in ParadigmSheet:" + (string)ps.name() + " is right connected to more than one FCO";
				throw udm_exception(error);
			}
			MetaGME::EquivRight derivedc = *(derivedcs.begin());
			MetaGME::BaseFCO derived = derivedc.srcFCO_end();

			AddEquivalence(derived, base);
		}
	}

	// Transfering Folder Equivalence association
	{
		vector<MetaGME::SameFolder> connects = ps.SameFolder_kind_children();
		for(vector<MetaGME::SameFolder>::iterator i = connects.begin(); i!=connects.end(); i++)
		{	MetaGME::SameFolder contain = *i;
			set<MetaGME::SameFolderLeft> basecs = contain.dstFolder();
			if(basecs.size() == 0)
			{	//Error
				string error = "Error: SameFolder object "+(string)contain.position()+ " in ParadigmSheet:" + (string)ps.name() + " is not left connected to an Folder";
				throw udm_exception(error);
			}
			if(basecs.size() > 1)
			{	//Error
				string error = "Error: SameFolder object "+(string)contain.position()+ " in ParadigmSheet:" + (string)ps.name() + " is left connected to more than one Folder";
				throw udm_exception(error);
			}
			MetaGME::SameFolderLeft basec = *(basecs.begin());
			MetaGME::BaseFolder base = basec.dstFolder_end();
			
			set<MetaGME::SameFolderRight> derivedcs = contain.srcFolder();
			if(derivedcs.size() == 0)
			{	//Error
				string error = "Error: SameFolder object "+(string)contain.position()+ " in ParadigmSheet:" + (string)ps.name() + " is not right connected to an Folder";
				throw udm_exception(error);
			}
			if(derivedcs.size() > 1)
			{	//Error
				string error = "Error: SameFolder object "+(string)contain.position()+ " in ParadigmSheet:" + (string)ps.name() + " is right connected to more than one Folder";
				throw udm_exception(error);
			}
			MetaGME::SameFolderRight derivedc = *(derivedcs.begin());
			MetaGME::BaseFolder derived = derivedc.srcFolder_end();
			
			AddEquivalence(derived, base);
			
		}
	}
}

void MetaGME2UML::AddEquivalence(MetaGME::ContainedInPar ms, MetaGME::ContainedInPar sl)
{
	MetaGME::ContainedInPar master = GetRefMetaGMEObject(ms);
	MetaGME::ContainedInPar slave  = GetRefMetaGMEObject(sl);

	MetaGME::ContainedInPar newslave;

	//string sn = slave.name();
	//string mn = master.name();

	// for the case where the same object O is to the left of 2 equivalence relations;
	while(equivMap.find(slave) != equivMap.end())
	{	newslave = master;
		master = equivMap[slave];
		slave = newslave;
		//sn = slave.name();
		//mn = master.name();
	}
	if(equivMap.find(master) == equivMap.end())
	{	equivMap[slave] = master;
		// search for slave in Map value and replace with master
		vector<MetaGME::ContainedInPar> fcos;
		for(map_CIP2CIP::iterator emi=equivMap.begin(); emi!=equivMap.end(); emi++)
		{	if( (*emi).second == slave)
				fcos.push_back((*emi).first);
		}
		for(vector<MetaGME::ContainedInPar>::iterator fi = fcos.begin(); fi!=fcos.end(); fi++)
		{	equivMap[*fi] = master;
		}
	}
	else
	{	equivMap[slave] = equivMap[master];
	}
}

void MetaGME2UML::Phase2_CreateImplementationInterfaceInheritances()
{	
	CreateImplementationInterfaceInheritances(grf,urf);
		
}

void MetaGME2UML::CreateImplementationInterfaceInheritances(MetaGME::RootFolder grf, Udm::Object urf)
{	
	vector<MetaGME::RootFolder> rfs = grf.RootFolder_kind_children();
	for(vector<MetaGME::RootFolder>::iterator ri=rfs.begin(); ri!=rfs.end();ri++)
	{	MetaGME::RootFolder sf = *ri;
		CreateImplementationInterfaceInheritances(sf, urf);
	}
	
	vector<MetaGME::SheetFolder> sfs = grf.SheetFolder_kind_children();
	for(vector<MetaGME::SheetFolder>::iterator i=sfs.begin(); i!=sfs.end();i++)
	{	MetaGME::SheetFolder sf = *i;
		vector<MetaGME::ParadigmSheet> pss = sf.ParadigmSheet_kind_children();
		for(vector<MetaGME::ParadigmSheet>::iterator j=pss.begin(); j!=pss.end();j++)
		{	MetaGME::ParadigmSheet ps = *j;
			UmlGME::ClassDiagram cd = mapMetaGME2UmlGME_Paradigm[ps];
			CreateImplementationInterfaceInheritances(ps,cd);
		}
	}
	
	vector<MetaGME::ParadigmSheet> pss = grf.ParadigmSheet_kind_children();
	for(vector<MetaGME::ParadigmSheet>::iterator j=pss.begin(); j!=pss.end();j++)
	{	MetaGME::ParadigmSheet ps = *j;
		UmlGME::ClassDiagram cd = mapMetaGME2UmlGME_Paradigm[ps];
		CreateImplementationInterfaceInheritances(ps,cd);
	}

}

void MetaGME2UML::CreateImplementationInterfaceInheritances(MetaGME::ParadigmSheet ps, UmlGME::ClassDiagram cd)
{	
	// Transfering Interface and Implementation Inheritance
	{	vector<MetaGME::InterfaceInheritance> connects = ps.InterfaceInheritance_kind_children();
		for(vector<MetaGME::InterfaceInheritance>::iterator i = connects.begin(); i!=connects.end(); i++)
		{	MetaGME::InterfaceInheritance contain = *i;
			set<MetaGME::BaseIntInheritance> basecs = contain.bintsrc();
			if(basecs.size() == 0)
			{	//Error
				string error = "Error: Interface inheritance object "+(string)contain.position()+ " in ParadigmSheet:" + (string)ps.name() + " is not connected to a base class";
				throw udm_exception(error);
			}
			if(basecs.size() > 1)
			{	//Error
				string error = "Error: Interface inheritance object "+(string)contain.position()+ " in ParadigmSheet:" + (string)ps.name() + " is not connected to more than one base class";
				throw udm_exception(error);
			}
			MetaGME::BaseIntInheritance basec = *(basecs.begin());
			set<MetaGME::DerivedIntInheritance> derivedcs = contain.dintdst();
			MetaGME::BaseFCO base = basec.bintsrc_end();
			
			base = MetaGME::BaseFCO::Cast(GetRefMetaGMEObject(base));

			//UmlGME::Class cbase = UmlGME::Class::Cast(mapMetaGME2UmlGME_Class[base]);

			for(set<MetaGME::DerivedIntInheritance>::iterator j=derivedcs.begin();j!=derivedcs.end();j++)
			{	MetaGME::DerivedIntInheritance derivedc = *j;
				MetaGME::BaseFCO derived = derivedc.dintdst_end();

				derived = MetaGME::BaseFCO::Cast(GetRefMetaGMEObject(derived));
				
				//UmlGME::Class cderived = UmlGME::Class::Cast(mapMetaGME2UmlGME_Class[derived]);
				MakeImplIntf(base, derived, false, true);

			}
		}
	}

	// Transfering Interface and Implementation Inheritance
	{	vector<MetaGME::ImplementationInheritance> connects = ps.ImplementationInheritance_kind_children();
		for(vector<MetaGME::ImplementationInheritance>::iterator i = connects.begin(); i!=connects.end(); i++)
		{	MetaGME::ImplementationInheritance contain = *i;
			set<MetaGME::BaseImpInheritance> basecs = contain.bimplsrc();
			if(basecs.size() == 0)
			{	//Error
				string error = "Error: Interface inheritance object "+(string)contain.position()+ " in ParadigmSheet:" + (string)ps.name() + " is not connected to a base class";
				throw udm_exception(error);
			}
			if(basecs.size() > 1)
			{	//Error
				string error = "Error: Interface inheritance object "+(string)contain.position()+ " in ParadigmSheet:" + (string)ps.name() + " is not connected to more than one base class";
				throw udm_exception(error);
			}
			MetaGME::BaseImpInheritance basec = *(basecs.begin());
			set<MetaGME::DerivedImpInheritance> derivedcs = contain.dimpldst();
			MetaGME::BaseFCO base = basec.bimplsrc_end();
			
			base = MetaGME::BaseFCO::Cast(GetRefMetaGMEObject(base));

			//UmlGME::Class cbase = UmlGME::Class::Cast(mapMetaGME2UmlGME_Class[base]);

			for(set<MetaGME::DerivedImpInheritance>::iterator j=derivedcs.begin();j!=derivedcs.end();j++)
			{	MetaGME::DerivedImpInheritance derivedc = *j;
				MetaGME::BaseFCO derived = derivedc.dimpldst_end();

				derived = MetaGME::BaseFCO::Cast(GetRefMetaGMEObject(derived));
				
				//UmlGME::Class cderived = UmlGME::Class::Cast(mapMetaGME2UmlGME_Class[derived]);
				MakeImplIntf(base, derived, true, false);

			}
		}
	}
}

MetaGME::ContainedInPar MetaGME2UML::GetRefMetaGMEObject(MetaGME::ContainedInPar base)
{	if(base == Udm::null )
	{	//Error
		string error = "Error: Function GetRefMetaGMEObject called with a null object";
		throw udm_exception(error);
	}
	
	Uml::Class basetype = base.type();

//	std::string typeName= basetype.name();
//	std::string name= MetaGME::BaseFCO::Cast( base).name();;

	if(!Udm::IsDerivedFrom(base.type(), MetaGME::ProxyBase::meta) && basetype != MetaGME::FolderProxy::meta)
		return base;

	Uml::Class nullClass( NULL);
	Udm::Object::AssociationInfo refinfo( nullClass);
//	Udm::Object::uniqueId_type uid= refinfo.clsAssociation.uniqueId();

	refinfo.strSrcRoleName = "src";
	refinfo.strDstRoleName = "ref";
	set<Udm::Object> objs = base.GetAdjacentObjects(MetaGME::ContainedInPar::meta, refinfo);
	if(objs.size() != 1)
	{	//Error
		string basename, sheetname;
		base.GetStrValue("name", basename); sheetname = MetaGME::ParadigmSheet::Cast(base.parent()).name();
		string error = "Error: Object "+ basename + " in ParadigmSheet:" + sheetname + " is a NULL reference";
		throw udm_exception(error);
	}
	return MetaGME::ContainedInPar::Cast(*objs.begin());
}

void MetaGME2UML::MakeIntImplDiagram()
{	if(!umlGMEimplinterfaceDiagram)
	{	umlGMEimplinterfaceDiagram = UmlGME::ClassDiagram::Create(urf);
		umlGMEimplinterfaceDiagram.name() = "zC_Interface_and_Implementation_Inheritance_diagram";
	}
}

void MetaGME2UML::MakeImplIntf(MetaGME::BaseFCO base, MetaGME::BaseFCO derived, bool impl, bool intf)
{	
	MakeIntImplDiagram();
	UmlGME::Class cbase, cderived;
	GetUMLCounterParts(base, derived, cbase, cderived);

	// Get Base impl and intf
	UmlGME::Class cbaseimpl, cbaseintf;
	UmlGME::Class cbaseref;

	map< UmlGME::ClassBase, pair< UmlGME::Class, UmlGME::Class> >::iterator bfi = mapUmlClass2ImplInheritBases.find(cbase);
	if(bfi == mapUmlClass2ImplInheritBases.end())
	{	cbaseimpl = UmlGME::Class::Create(umlGMEimplinterfaceDiagram);
		cbaseimpl.name() = (string)cbase.name() + "ImplementationBase";
		cbaseimpl.position() = MakePosition(iox,ioy,iox,ioy); iox += 250;
		cbaseimpl.IsAbstract() = true;
		
		cbaseintf = UmlGME::Class::Create(umlGMEimplinterfaceDiagram);
		cbaseintf.name() = (string)cbase.name() + "InterfaceBase";
		cbaseintf.position() = MakePosition(iox,ioy,iox,ioy); iox -= 200; ioy += 150;
		cbaseintf.IsAbstract() = true;

		UmlGME::Inheritance cbimplinh = UmlGME::Inheritance::Create(umlGMEimplinterfaceDiagram);
		cbimplinh.position() = MakePosition(iox,ioy,iox,ioy); iox += 250;
		cbimplinh.name() = "";

		UmlGME::Inheritance cbintfinh = UmlGME::Inheritance::Create(umlGMEimplinterfaceDiagram);
		cbintfinh.position() = MakePosition(iox,ioy,iox,ioy); iox -= 175; ioy += 150;
		cbintfinh.name() = "";
		
		cbaseref = cbase;
		cbaseref.position() = MakePosition(iox,ioy,iox,ioy); iox -= 125; ioy += 150;

		UmlGME::Base cbc = UmlGME::Base::Create(umlGMEimplinterfaceDiagram);
		cbc.dstInheritance_end() = cbimplinh;
		cbc.srcClassBase_end() = cbaseimpl;

		UmlGME::Sub csub = UmlGME::Sub::Create(umlGMEimplinterfaceDiagram);
		csub.srcInheritance_end() = cbimplinh;
		csub.dstClassBase_end() = cbaseref;

		cbc = UmlGME::Base::Create(umlGMEimplinterfaceDiagram);
		cbc.dstInheritance_end() = cbintfinh;
		cbc.srcClassBase_end() = cbaseintf;

		csub = UmlGME::Sub::Create(umlGMEimplinterfaceDiagram);
		csub.srcInheritance_end() = cbintfinh;
		csub.dstClassBase_end() = cbaseref;
		
		pair<UmlGME::Class, UmlGME::Class> pr;
		pr.first = cbaseimpl; pr.second = cbaseintf;
		mapUmlClass2ImplInheritBases[cbase] = pr;
	}
	else
	{	cbaseimpl = (*bfi).second.first;
		cbaseintf = (*bfi).second.second;
	}

	// Get Derived impl and intf
	UmlGME::Class cderivedimpl, cderivedintf;
	UmlGME::Class cderivedref;

	map< UmlGME::ClassBase, pair< UmlGME::Class, UmlGME::Class> >::iterator dfi = mapUmlClass2ImplInheritBases.find(cderived);
	if(dfi == mapUmlClass2ImplInheritBases.end())
	{	cderivedimpl = UmlGME::Class::Create(umlGMEimplinterfaceDiagram);
		cderivedimpl.name() = (string)cderived.name() + "ImplementationBase";
		cderivedimpl.position() = MakePosition(iox,ioy,iox,ioy); iox += 250;
		cderivedimpl.IsAbstract() = true;

		cderivedintf = UmlGME::Class::Create(umlGMEimplinterfaceDiagram);
		cderivedintf.name() = (string)cderived.name() + "InterfaceBase";
		cderivedintf.position() = MakePosition(iox,ioy,iox,ioy); iox -= 200; ioy +=150;
		cderivedintf.IsAbstract() = true;

		UmlGME::Inheritance cdimplinh = UmlGME::Inheritance::Create(umlGMEimplinterfaceDiagram);
		cdimplinh.position() = MakePosition(iox,ioy,iox,ioy); iox += 250;
		cdimplinh.name() = "";

		UmlGME::Inheritance cdintfinh = UmlGME::Inheritance::Create(umlGMEimplinterfaceDiagram);
		cdintfinh.position() = MakePosition(iox,ioy,iox,ioy); iox -= 175; ioy += 150;
		cdintfinh.name() = "";

		cderivedref = cderived;
		cderivedref.position() = MakePosition(iox,ioy,iox,ioy); iox -= 125; ioy +=150;

		UmlGME::Base cbc = UmlGME::Base::Create(umlGMEimplinterfaceDiagram);
		cbc.dstInheritance_end() = cdimplinh;
		cbc.srcClassBase_end() = cderivedimpl;

		UmlGME::Sub csub = UmlGME::Sub::Create(umlGMEimplinterfaceDiagram);
		csub.srcInheritance_end() = cdimplinh;
		csub.dstClassBase_end() = cderivedref;

		cbc = UmlGME::Base::Create(umlGMEimplinterfaceDiagram);
		cbc.dstInheritance_end() = cdintfinh;
		cbc.srcClassBase_end() = cderivedintf;

		csub = UmlGME::Sub::Create(umlGMEimplinterfaceDiagram);
		csub.srcInheritance_end() = cdintfinh;
		csub.dstClassBase_end() = cderivedref;
		
		pair<UmlGME::Class, UmlGME::Class> pr;
		pr.first = cderivedimpl; pr.second = cderivedintf;
		mapUmlClass2ImplInheritBases[cderived] = pr;
	}
	else
	{	cderivedimpl = (*dfi).second.first;
		cderivedintf = (*dfi).second.second;
	}

	if(impl)
	{	UmlGME::Inheritance cinh = UmlGME::Inheritance::Create(umlGMEimplinterfaceDiagram);
		
		int bx=0, by=0, dx=0, dy=0;
		GetPosAttr((string)cbaseimpl.position(), "UML", bx, by);
		GetPosAttr((string)cderivedimpl.position(), "UML", dx, dy);

		cinh.position() = MakePosition(50, by + (dy-by)/2 , 50 , by + (dy-by)/2);
		UmlGME::Base cbc = UmlGME::Base::Create(umlGMEimplinterfaceDiagram);
		cbc.dstInheritance_end() = cinh;
		cbc.srcClassBase_end() = cbaseimpl;

		UmlGME::Sub csub = UmlGME::Sub::Create(umlGMEimplinterfaceDiagram);
		csub.srcInheritance_end() = cinh;
		csub.dstClassBase_end() = cderivedimpl;

		maphasbaseimpl[cderived] = true;
	}

	if(intf)
	{	UmlGME::Inheritance cinh = UmlGME::Inheritance::Create(umlGMEimplinterfaceDiagram);
		
		int bx=0, by=0, dx=0, dy=0;
		GetPosAttr((string)cbaseimpl.position(), "UML", bx, by);
		GetPosAttr((string)cderivedimpl.position(), "UML", dx, dy);

		cinh.position() = MakePosition(500, by + (dy-by)/2 , 500 , by + (dy-by)/2);
		UmlGME::Base cbc = UmlGME::Base::Create(umlGMEimplinterfaceDiagram);
		cbc.dstInheritance_end() = cinh;
		cbc.srcClassBase_end() = cbaseintf;

		UmlGME::Sub csub = UmlGME::Sub::Create(umlGMEimplinterfaceDiagram);
		csub.srcInheritance_end() = cinh;
		csub.dstClassBase_end() = cderivedintf;

		maphasbaseintf[cderived] = true;
	}
}


// Get the UML counterparts
void MetaGME2UML::GetUMLCounterParts(MetaGME::BaseFCO base, MetaGME::BaseFCO derived, UmlGME::Class &cbase, UmlGME::Class &cderived)
{	MetaGME::BaseFCO ebase, ederived;
	if(mapMetaGME2UmlGME_Class.find(base) == mapMetaGME2UmlGME_Class.end())
	{	if(equivMap.find(base) != equivMap.end())	
		{	ebase = MetaGME::BaseFCO::Cast(equivMap[base]);
			if(mapMetaGME2UmlGME_Class.find(ebase) != mapMetaGME2UmlGME_Class.end())
			{	mapMetaGME2UmlGME_Class[base] = mapMetaGME2UmlGME_Class[ebase];
				//RecurseOnBases(base);
			}
			else
			{	cbase = UmlGME::Class::Create(umlGMEimplinterfaceDiagram);
				cbase.name() = base.name();
				mapMetaGME2UmlGME_Class[base]  = cbase;
				mapMetaGME2UmlGME_Class[ebase] = cbase;
				RecurseOnBases(ebase);
			}
			
		}
		else
		{	cbase = UmlGME::Class::Create(umlGMEimplinterfaceDiagram);
			cbase.name() = base.name();
			mapMetaGME2UmlGME_Class[base] = cbase;
			RecurseOnBases(base);
		}
		
	}
	else 
	{	cbase = UmlGME::Class::Cast(mapMetaGME2UmlGME_Class[base]);
	}

	if(mapMetaGME2UmlGME_Class.find(derived) == mapMetaGME2UmlGME_Class.end())
	{	cderived = UmlGME::Class::Create(umlGMEimplinterfaceDiagram);
		cderived.name() = derived.name();
		mapMetaGME2UmlGME_Class[derived] = cderived;
	}
	else
	{	cderived = UmlGME::Class::Cast(mapMetaGME2UmlGME_Class[derived]);
	}
}

void MetaGME2UML::RecurseOnBases(MetaGME::BaseFCO base)
{	// recurse on base
	{	set<MetaGME::DerivedInheritance> dihs = base.dsrc();
		for(set<MetaGME::DerivedInheritance>::iterator i=dihs.begin(); i!=dihs.end(); i++)
		{	MetaGME::DerivedInheritance dih = *i;
			MetaGME::Inheritance inh = dih.dsrc_end();
			set<MetaGME::BaseInheritance> bihs = inh.bsrc();
			for(set<MetaGME::BaseInheritance>::iterator i=bihs.begin(); i!=bihs.end(); i++)
			{	MetaGME::BaseInheritance bih = *i;
				MetaGME::BaseFCO basebase = bih.bsrc_end();
				MakeImplIntf(basebase, base, true, true);
			}

		}
		
	}
	// Note: add stuff here for reference inheritance

	vector<MetaGME::ContainedInPar> fcos;
	for(map_CIP2CIP::iterator emi=equivMap.begin(); emi!=equivMap.end(); emi++)
	{	if( (*emi).second == base)
			fcos.push_back((*emi).first);
	}
	for(vector<MetaGME::ContainedInPar>::iterator fi = fcos.begin(); fi!=fcos.end(); fi++)
	{	RecurseOnBases(MetaGME::BaseFCO::Cast(*fi));
	}
}

void MetaGME2UML::Phase3_CreateClasses()
{	CreateClasses(grf,urf);
}

void MetaGME2UML::CreateClasses(MetaGME::RootFolder grf, Udm::Object urf)
{	
	vector<MetaGME::RootFolder> rfs = grf.RootFolder_kind_children();
	for(vector<MetaGME::RootFolder>::iterator ri=rfs.begin(); ri!=rfs.end();ri++)
	{	MetaGME::RootFolder sf = *ri;
		CreateClasses(sf, urf);
	}
	
	vector<MetaGME::SheetFolder> sfs = grf.SheetFolder_kind_children();
	for(vector<MetaGME::SheetFolder>::iterator i=sfs.begin(); i!=sfs.end();i++)
	{	MetaGME::SheetFolder sf = *i;
		vector<MetaGME::ParadigmSheet> pss = sf.ParadigmSheet_kind_children();
		for(vector<MetaGME::ParadigmSheet>::iterator j=pss.begin(); j!=pss.end();j++)
		{	MetaGME::ParadigmSheet ps = *j;
			UmlGME::ClassDiagram cd = mapMetaGME2UmlGME_Paradigm[ps];
			CreateClasses(ps,cd);
		}
	}
	
	vector<MetaGME::ParadigmSheet> pss = grf.ParadigmSheet_kind_children();
	for(vector<MetaGME::ParadigmSheet>::iterator j=pss.begin(); j!=pss.end();j++)
	{	MetaGME::ParadigmSheet ps = *j;
		UmlGME::ClassDiagram cd = mapMetaGME2UmlGME_Paradigm[ps];
		CreateClasses(ps,cd);
	}
}

void MetaGME2UML::CreateClasses(MetaGME::ParadigmSheet ps, UmlGME::ClassDiagram cd)
{	vector<MetaGME::ContainedInPar> cips = ps.ContainedInPar_kind_children();
	for(vector<MetaGME::ContainedInPar>::iterator i=cips.begin(); i!=cips.end();i++)
	{	MetaGME::ContainedInPar cip = *i;
		
		Uml::Class cipType = cip.type();
		string type = cipType.name();
		if(Udm::IsDerivedFrom(cipType, MetaGME::ProxyBase::meta) || type == "FolderProxy")
			continue;
		
		if(equivMap.find(cip)!= equivMap.end())
		{	continue;
		}
		
		int setmaps = false;

		UmlGME::Class cl, impl, intf;
		if(mapMetaGME2UmlGME_Class.find(cip)!=mapMetaGME2UmlGME_Class.end())
		{	cl = UmlGME::Class::Cast(mapMetaGME2UmlGME_Class[cip]);
			impl = mapUmlClass2ImplInheritBases[cl].first;
			intf = mapUmlClass2ImplInheritBases[cl].second;
		}
		else
		{	cl = UmlGME::Class::Create(cd);
			impl = cl; intf = cl;
			pair<UmlGME::Class, UmlGME::Class> pr;
			pr.first = impl; pr.second = intf;
			mapUmlClass2ImplInheritBases[cl] = pr;
			implMap[cip] = cl;
			intfMap[cip] = cl;
			reguMap[cip] = cl;
			setmaps = true;
			cl.position() = MakePosAttr(cip.position(), attmap);
			mapMetaGME2UmlGME_Class[cip] = cl;
		}

		if(type == "Folder")
		{	MetaGME::Folder mo = MetaGME::Folder::Cast(cip);
			cl.name() = mo.name();
			cl.Stereotype() = "Folder";
			impl.Stereotype() = "Folder";
			intf.Stereotype() = "Folder";
			cl.IsAbstract() = false;
			cl.Attributes() = "name : String[0..1]\n";
			if(mo.InRootFolder())	CreateRootFolderAssociation(intf);
		}
		else // either Model, Atom, FCO, Set, Reference, Connection
		{	MetaGME::BaseFCO mo = MetaGME::BaseFCO::Cast(cip);
			cl.name() = mo.name();
			cl.IsAbstract() = mo.IsAbstract();
			cl.Stereotype() = type;
			impl.Stereotype() = type;
			intf.Stereotype() = type;
			if(mo.InRootFolder())	CreateRootFolderAssociation(intf);
			AddAttribute(mo,impl);
		}

		vector<MetaGME::ContainedInPar> fcos;
		for(map_CIP2CIP::iterator emi=equivMap.begin(); emi!=equivMap.end(); emi++)
		{	if( (*emi).second == cip)
				fcos.push_back((*emi).first);
		}
		for(vector<MetaGME::ContainedInPar>::iterator fi = fcos.begin(); fi!=fcos.end(); fi++)
		{	MetaGME::ContainedInPar slave = *fi;
			mapMetaGME2UmlGME_Class[slave] = cl;
			if(setmaps && MetaGME::ParadigmSheet::Cast(slave.parent()) == MetaGME::ParadigmSheet::Cast(cip.parent()) )
			{	implMap[slave] = implMap[cip];
				intfMap[slave] = intfMap[cip];
				reguMap[slave] = reguMap[cip];
			}
			if(Udm::IsDerivedFrom(slave.type(), MetaGME::BaseFCO::meta))
			{	AddAttribute(MetaGME::BaseFCO::Cast(slave),impl);
			}
		}
	}
}

void MetaGME2UML::Phase4_CreateReferences()
{	
	CreateReferences(grf,urf);
		
}

void MetaGME2UML::CreateReferences(MetaGME::RootFolder grf, Udm::Object urf)
{	
	vector<MetaGME::RootFolder> rfs = grf.RootFolder_kind_children();
	for(vector<MetaGME::RootFolder>::iterator ri=rfs.begin(); ri!=rfs.end();ri++)
	{	MetaGME::RootFolder sf = *ri;
		CreateReferences(sf, urf);
	}
	
	vector<MetaGME::SheetFolder> sfs = grf.SheetFolder_kind_children();
	for(vector<MetaGME::SheetFolder>::iterator i=sfs.begin(); i!=sfs.end();i++)
	{	MetaGME::SheetFolder sf = *i;
		vector<MetaGME::ParadigmSheet> pss = sf.ParadigmSheet_kind_children();
		for(vector<MetaGME::ParadigmSheet>::iterator j=pss.begin(); j!=pss.end();j++)
		{	MetaGME::ParadigmSheet ps = *j;
			UmlGME::ClassDiagram cd = mapMetaGME2UmlGME_Paradigm[ps];
			CreateReferences(ps,cd);
		}
	}
	
	vector<MetaGME::ParadigmSheet> pss = grf.ParadigmSheet_kind_children();
	for(vector<MetaGME::ParadigmSheet>::iterator j=pss.begin(); j!=pss.end();j++)
	{	MetaGME::ParadigmSheet ps = *j;
		UmlGME::ClassDiagram cd = mapMetaGME2UmlGME_Paradigm[ps];
		CreateReferences(ps,cd);
	}

}

void MetaGME2UML::CreateReferences(MetaGME::ParadigmSheet ps, UmlGME::ClassDiagram cd)
{	
	vector<MetaGME::ContainedInPar> cips = ps.ContainedInPar_kind_children();
//	int sz= cips.size();
	for(vector<MetaGME::ContainedInPar>::iterator i=cips.begin(); i!=cips.end();i++)
	{	MetaGME::ContainedInPar cip = *i;
	std::string cipname= cip.name();
		Uml::Class cipType = cip.type();
//		string type = cipType.name();
		if(!Udm::IsDerivedFrom(cipType, MetaGME::ProxyBase::meta) && ( cipType != MetaGME::FolderProxy::meta))
			continue;
		
		MetaGME::ContainedInPar cipO = GetRefMetaGMEObject(cip);
		if(mapMetaGME2UmlGME_Class.find(cipO)!=mapMetaGME2UmlGME_Class.end())
		{	mapMetaGME2UmlGME_Class[cip] = mapMetaGME2UmlGME_Class[cipO];
			
			MetaGME::ContainedInPar cipb = cip;
			UmlGME::Class rcl = UmlGME::Class::Cast(mapMetaGME2UmlGME_Class[cipO]);
			if(cipb == Udm::null || rcl == Udm::null)
				MESSAGE_OUT("NULL || NULL");
			AddAttribute(cipb,rcl);
			//continue;
		}
		continue;
		
		// code below this is not executed as of now
		/*
		UmlGME::ClassCopy cl = UmlGME::ClassCopy::Create(cd);
		implMap[cip] = cl;
		intfMap[cip] = cl;
		reguMap[cip] = cl;

		if(type == "ModelProxy")
		{	MetaGME::ModelProxy mo = MetaGME::ModelProxy::Cast(cip);
			cl.name() = mo.name();
			cl.position() = MakePosAttr(mo.position(), attmap);
			MetaGME::Model m = mo.ref();
			map_CIP2CB::iterator mf = mapMetaGME2UmlGME_Class.find(m);
			if(mf==mapMetaGME2UmlGME_Class.end())
			{	//Error
				string error = "Error: ModelProxy:" + (string)mo.name() + " in ParadigmSheet:" + (string)ps.name() + " refers to NULL";
				throw udm_exception(error);
			}
			UmlGME::Class cls = UmlGME::Class::Cast(mf->second);
			cl.ref() = cls;
			AddAttribute(mo,cls);
			mapMetaGME2UmlGME_Class[mo] = cl;
		}
		else if(type == "AtomProxy")
		{	MetaGME::AtomProxy mo = MetaGME::AtomProxy::Cast(cip);
			cl.name() = mo.name();
			cl.position() = MakePosAttr(mo.position(), attmap);
			MetaGME::Atom m = MetaGME::Atom::Cast(mo.ref());
			map_CIP2CB::iterator mf = mapMetaGME2UmlGME_Class.find(m);
			if(mf==mapMetaGME2UmlGME_Class.end())
			{	//Error
				string error = "Error: AtomProxy:" + (string)mo.name() + " in ParadigmSheet:" + (string)ps.name() + " refers to NULL";
				throw udm_exception(error);
			}
			UmlGME::Class cls = UmlGME::Class::Cast(mf->second);
			cl.ref() = cls;
			AddAttribute(mo,cls);
			mapMetaGME2UmlGME_Class[mo] = cl;
		}
		else if(type == "FolderProxy")
		{	MetaGME::FolderProxy mo = MetaGME::FolderProxy::Cast(cip);
			cl.name() = mo.name();
			cl.position() = MakePosAttr(mo.position(), attmap);
			MetaGME::Folder m = MetaGME::Folder::Cast(mo.ref());
			map_CIP2CB::iterator mf = mapMetaGME2UmlGME_Class.find(m);
			if(mf==mapMetaGME2UmlGME_Class.end())
			{	//Error
				string error = "Error: FolderProxy:" + (string)mo.name() + " in ParadigmSheet:" + (string)ps.name() + " refers to NULL";
				throw udm_exception(error);
			}
			UmlGME::Class cls = UmlGME::Class::Cast(mf->second);
			cl.ref() = cls;
			mapMetaGME2UmlGME_Class[mo] = cl;
		}
		else if(type == "FCOProxy")
		{	MetaGME::FCOProxy mo = MetaGME::FCOProxy::Cast(cip);
			cl.name() = mo.name();
			cl.position() = MakePosAttr(mo.position(), attmap);
			MetaGME::FCO m = MetaGME::FCO::Cast(mo.ref());
			map_CIP2CB::iterator mf = mapMetaGME2UmlGME_Class.find(m);
			if(mf==mapMetaGME2UmlGME_Class.end())
			{	//Error
				string error = "Error: FCOProxy:" + (string)mo.name() + " in ParadigmSheet:" + (string)ps.name() + " refers to NULL";
				throw udm_exception(error);
			}
			UmlGME::Class cls = UmlGME::Class::Cast(mf->second);
			cl.ref() = cls;
			AddAttribute(mo,cls);
			mapMetaGME2UmlGME_Class[mo] = cl;
		}
		else if(type == "SetProxy")
		{	MetaGME::SetProxy mo = MetaGME::SetProxy::Cast(cip);
			cl.name() = mo.name();
			cl.position() = MakePosAttr(mo.position(), attmap);
			MetaGME::Set m = MetaGME::Set::Cast(mo.ref());
			map_CIP2CB::iterator mf = mapMetaGME2UmlGME_Class.find(m);
			if(mf==mapMetaGME2UmlGME_Class.end())
			{	//Error
				string error = "Error: SetProxy:" + (string)mo.name() + " in ParadigmSheet:" + (string)ps.name() + " refers to NULL";
				throw udm_exception(error);
			}
			UmlGME::Class cls = UmlGME::Class::Cast(mf->second);
			cl.ref() = cls;
			AddAttribute(mo,cls);
			mapMetaGME2UmlGME_Class[mo] = cl;
		}
		else if(type == "ReferenceProxy")
		{	MetaGME::ReferenceProxy mo = MetaGME::ReferenceProxy::Cast(cip);
			cl.name() = mo.name();
			cl.position() = MakePosAttr(mo.position(), attmap);
			MetaGME::Reference m = MetaGME::Reference::Cast(mo.ref());
			map_CIP2CB::iterator mf = mapMetaGME2UmlGME_Class.find(m);
			if(mf==mapMetaGME2UmlGME_Class.end())
			{	//Error
				string error = "Error: ReferenceProxy:" + (string)mo.name() + " in ParadigmSheet:" + (string)ps.name() + " refers to NULL";
				throw udm_exception(error);
			}
			UmlGME::Class cls = UmlGME::Class::Cast(mf->second);
			cl.ref() = cls;
			AddAttribute(mo,cls);
			mapMetaGME2UmlGME_Class[mo] = cl;
		}
		else if(type == "ConnectionProxy")
		{	UmlGME::ClassCopy cl = UmlGME::ClassCopy::Create(cd);
			MetaGME::ConnectionProxy mo = MetaGME::ConnectionProxy::Cast(cip);
			cl.name() = mo.name();
			cl.position() = MakePosAttr(mo.position(), attmap);
			MetaGME::Connection m = MetaGME::Connection::Cast(mo.ref());
			map_CIP2CB::iterator mf = mapMetaGME2UmlGME_Class.find(m);
			if(mf==mapMetaGME2UmlGME_Class.end())
			{	//Error
				string error = "Error: ConnectionProxy:" + (string)mo.name() + " in ParadigmSheet:" + (string)ps.name() + " refers to NULL";
				throw udm_exception(error);
			}
			UmlGME::Class cls = UmlGME::Class::Cast(mf->second);
			cl.ref() = cls;
			AddAttribute(mo,cls);
			mapMetaGME2UmlGME_Class[mo] = cl;
		}
		*/
		
	}
}