/*
Copyright (c) Vanderbilt University, 2000-2001
ALL RIGHTS RESERVED
Vanderbilt University disclaims all warranties with regard to this
software, including all implied warranties of merchantability
and fitness.  In no event shall Vanderbilt University be liable for
any special, indirect or consequential damages or any damages
whatsoever resulting from loss of use, data or profits, whether
in an action of contract, negligence or other tortious action,
arising out of or in connection with the use or performance of
this software.
*/


// UdmApp.cpp: implementation of the CUdmApp class.
// This file was automatically generated as UdmApp.cpp
// by UDM Interpreter Wizard on Monday, May 13, 2002 13:45:42

// Tihamr Levendovszky 05-13-02

#include "stdafx.h"
#include <afxdlgs.h> // For CFileDialog
#include "resource.h"
#include "OldNewDialog.h"

//#include <iostream.h>
#include <atlbase.h>
#include <comdef.h>

#include <time.h>
#include "UdmStatic.h"
#include "UdmDom.h"
#include "UdmGME.h"
#include "UmlExt.h"
#include "UdmUtil.h"

#include "UdmApp.h"
#include "UdmConfig.h"

#include "UMLModelTransformer.h"
#include "GReATConfig.h"
#include "umt2cfg.h"

#include "umt_exception.h"

#include "UniqueFunctor.h"

extern void dummy(void); // Dummy function for UDM meta initialization

char* GetRelativeFilename(const char *currentDirectory,const char *absoluteFilename);

#include "TempDNProcessor.h"
//#include "GReATConfigUsage.h"

// Checking for correct config file.  
void checkConfig( const UMLModelTransformer::Configuration& cnf, std::string& trans, std::string& udmprj) throw ( udm_exception)
{ 
	// The comments in quotes refer to the names of constraints in the UMT meta paradigm.
	// 1 "OnlyOneMetaInformation"
	vector<UMLModelTransformer::MetaInformation> mis = cnf.MetaInformation_kind_children();
	if(mis.size() == 0 || mis.size() > 1)
	{	//Error: The configuration file doesn't contain a meta information
		throw udm_exception("Config Error: The configuration model either doesnt contain meta information or it contains more than one");
	}
	UMLModelTransformer::MetaInformation mi = *mis.begin();
	// 2 Transformation file not null
	trans = mi.TransformationFile();
	if(trans == "") 
	{	//Error: Transformation GR file information is missing
		throw udm_exception("Config Error: Transformation GR file information is missing. Please run the GenerateGR interpreter");
	}
	// 3 Udm project file not null
	udmprj = mi.UdmProjectFile();
	if(udmprj == "")
	{	//Error: Udm Project file information is missing
		throw udm_exception("Config Error: Udm Project file information is missing. Please run the UML2XML interpreter");
	}
	// 4 "OnlyOneStartRule"
	vector<UMLModelTransformer::StartRule> srs = cnf.StartRule_kind_children();
	if(srs.size() == 0 || mis.size() > 1)
	{	//Error: The configuration file doesn't contain a start rule
		string msg = "Config Error: ";
		msg += "Configuration: "+(string)cnf.name() + " doesnt contain a StartRule or or it contains more than one.";
		throw udm_exception(msg);			
	}
	// 5 "NullStartRule"
	UMLModelTransformer::StartRule sr = *(srs.begin());
	UMLModelTransformer::Expression sre = sr.ref();
	if(sre == Udm::null)
	{	string msg = "Config Error: ";
		msg += "Configuration: "+(string)cnf.name() + "->StartRule:"+(string)sr.name()+" is a null reference, please make it refer to a valid transformaition.";
		throw udm_exception(msg);
	}
	// 6 "TransformationInputError"
	vector<UMLModelTransformer::In> ins = sre.In_kind_children();
	for(vector<UMLModelTransformer::In>::iterator ini = ins.begin(); ini != ins.end(); ini++)
	{	const UMLModelTransformer::In &in = *ini;
		set<UMLModelTransformer::TransformationInput> tis = in.srcTransformationInput();
		if(tis.size() != 1)
		{	string msg = "Config Error: ";
			msg += "Configuration: "+(string)cnf.name() + "->StartRule: "+(string)sr.name()+"->InputPort: "+(string)in.name()+ " Has either zero or more than one FileObject association";
			throw udm_exception(msg);
		}
	}
	// 7 "UniqueFileTypeName"
	vector<UMLModelTransformer::FileType> fts = cnf.FileType_kind_children();
	UniqueAttrsFunctor uniqueNames( "name");
	uniqueNames= for_each( fts.begin(), fts.end(), uniqueNames);
	if ( false== uniqueNames.isUnique()) {
		string msg = "Config Error: ";
		msg += "Configuration: "+(string)cnf.name() + " contains duplicate file type names: " + uniqueNames.duplicated();
		throw udm_exception(msg);
	}
	// 8 "UniqueMetaName"
//	UniqueAttrsFunctor uniqueMetaNames( "MetaName");
//	uniqueMetaNames= for_each( fts.begin(), fts.end(), uniqueMetaNames);
//	if ( false== uniqueMetaNames.isUnique()) {
//		string msg = "Config Error: ";
//		msg += "Configuration: "+(string)cnf.name() + " contains duplicate meta names: " + uniqueMetaNames.duplicated();
//		throw udm_exception(msg);
//	}
	// 9 "NullMetaName"
	EmptyAttrFunctor eafMetaName( "MetaName");
	std::vector< UMLModelTransformer::FileType>::const_iterator itFindEmpty= std::find_if( fts.begin(), fts.end(), eafMetaName);
	if ( itFindEmpty != fts.end()) {
		string msg = "Config Error: ";
		msg += "Configuration: "+(string)cnf.name() + " contains empty meta name in file type: " + ( std::string)itFindEmpty->name();
		throw udm_exception(msg);
	}
	// 10 "NullRootClsName"	
	EmptyAttrFunctor eafRootClsName( "RootClassName");
	std::vector< UMLModelTransformer::FileType>::const_iterator itFindEmpty2= std::find_if( fts.begin(), fts.end(), eafRootClsName);
	if ( itFindEmpty2 != fts.end()) {
		string msg = "Config Error: ";
		msg += "Configuration: "+(string)cnf.name() + " contains empty root class name in file type: " + ( std::string)itFindEmpty2->name();
		throw udm_exception(msg);
	}
	// 11 "OnlyOneFileConnection" this constraint is fired on action, so we should not encounter this here.
	for( std::vector< UMLModelTransformer::FileType>::const_iterator it1= fts.begin(); it1!= fts.end(); ++it1) {
		const set< UMLModelTransformer::FileBind>& fileConns= (*it1).srcFileBind();
		if ( fileConns.size() > 1) {
			string msg = "Config Error: ";
			msg += "Configuration: "+(string)cnf.name() + "->FileType: " + ( std::string)it1->name() + "is connected to more than one files.";
			throw udm_exception(msg);
		}
	}
	// 12 "OnlyOneFileTypeConnect" this constraint is fired on action, so we should not encounter this here.
	std::vector< UMLModelTransformer::File> files= cnf.File_kind_children();
	for( std::vector< UMLModelTransformer::File>::const_iterator it2= files.begin(); it2!= files.end(); ++it2) {
		const set< UMLModelTransformer::FileBind>& fileConns= (*it2).dstFileBind();
		if ( fileConns.size() > 1) {
			string msg = "Config Error: ";
			msg += "Configuration: "+(string)cnf.name() + "->File: " + ( std::string)it2->name() + "is connected to more than one file types.";
			throw udm_exception(msg);
		}
	}
	// 13 "NullObjPath"
	for( std::vector< UMLModelTransformer::FileType>::const_iterator it3= fts.begin(); it3!= fts.end(); ++it3) {
		std::vector<UMLModelTransformer::FileObject> fos = (*it3).FileObject_kind_children();
		EmptyAttrFunctor eafObjPath( "ObjectPath");
		std::vector< UMLModelTransformer::FileObject>::const_iterator itFindEmpty3= std::find_if( fos.begin(), fos.end(), eafObjPath);
		if ( itFindEmpty3 != fos.end()) {
			string msg = "Config Error: ";
			msg += "Configuration: "+(string)cnf.name() + " contains empty object path in file type: " + ( std::string)itFindEmpty3->name();
			throw udm_exception(msg);
		}
	}
	// 14 "UniqueInPortName"
	UniqueAttrsFunctor uniqueInputPortNames( "name");
	uniqueInputPortNames= for_each( ins.begin(), ins.end(), uniqueInputPortNames);	// 'ins' comes from: 6 "TransformationInputError"
	if ( false== uniqueInputPortNames.isUnique()) {
		string msg = "Config Error: ";
		msg += "Configuration: "+(string)cnf.name() + "->StartRule: "+(string)sr.name()+"->InputPort name: " + uniqueInputPortNames.duplicated()+ " is not unique.";
		throw udm_exception(msg);
	}
}

/*********************************************************************************/
/* Initialization function. The framework calls it before preparing the backend. */
/* Initialize here the settings in the config static object.					 */
/* Return 0 if successful.														 */
/*********************************************************************************/
int CUdmApp::Initialize()
{
	// TODO: Your initialization code comes here...
	return 0;
}



/* 
Remarks to CUdmApp::UdmMain(...):
0.	The p_backend points to an already open backend, and the framework 
	closes it automatically. DO NOT OPEN OR CLOSE IT!
	To commit changes use p_backend->CommitEditSequence().
	To abort changes use p_backend->AbortEditSequence().
	To save changes to a different file use p_backend->SaveAs() or p_backend->CloseAs().

1.	Focus is the currently open model.

2.	The possible values for param (from GME DecoratorLib.h component_startmode_enum):
	GME_MAIN_START			=   0,
	GME_BROWSER_START		=   1,
	GME_CONTEXT_START		=   2,
	GME_EMBEDDED_START		=   3,
	GME_MENU_START			=  16,
	GME_BGCONTEXT_START		=  18,
	GME_ICON_START			=  32,
	METAMODEL_CHECK_SYNTAX	= 101

 3. The framework catches all the exceptions and reports the error in a message box,
	clean up and close the transactions aborting the changes. You can override this 
	behavior by catching udm_exception. Use udm_exception::what() to form an error 
	message.
*/

/***********************************************/
/* Main entry point for Udm-based Interpreter  */
/***********************************************/

void CUdmApp::UdmMain(
					 Udm::DataNetwork* p_backend,		// Backend pointer(already open!)
					 Udm::Object focusObject,			// Focus object
					 set<Udm::Object> selectedObjects,	// Selected objects
					 long param)						// Parameters
{	
	//try
	{			
		UMLModelTransformer::RootFolder rf = UMLModelTransformer::RootFolder::Cast(p_backend->GetRootObject());
		UMLModelTransformer::Configuration cnf;
		vector<UMLModelTransformer::Configuration> confs;
		
		// Get the configuration model
		if(focusObject != Udm::null && (string)focusObject.type().name() == "Configuration")
		{	cnf = UMLModelTransformer::Configuration::Cast(focusObject);
		}
		else
		{	vector<UMLModelTransformer::Configurations> cfolds = rf.Configurations_children();
			for(vector<UMLModelTransformer::Configurations>::iterator i=cfolds.begin(); i!=cfolds.end();i++)
			{	UMLModelTransformer::Configurations cfold = *i;
				vector<UMLModelTransformer::Configuration> cfgs = cfold.Configuration_kind_children();
				confs.insert(confs.end(), cfgs.begin(), cfgs.end());
			}
			if(confs.size() == 0)
			{	throw udm_exception("Config Error: Couldnt find any configuration model");
			}
			if(confs.size() > 1)
			{	throw udm_exception("Config Error: More than one configuration found. Please make the desired one the focused object and then press interepret");
			}
			cnf = *confs.begin();
		}
		
		string name = cnf.ConfigFile();
		if(name == "" || name[1] != ':')
		{	throw udm_exception("Config Error: No configuration file name. Run UMT Master interpreter.");
		}
		// Checking for correct config file.
		std::string trans;
		std::string udmprj;
		checkConfig( cnf, trans, udmprj);
		//
		string configDir;
		int ne = name.find_last_of('\\');
		configDir = name.substr(0,ne+1);
		if(CreateDirectory(configDir.c_str(), NULL)==0)
		{	int le = GetLastError();
			if(le != 183)
			{	LPVOID lpMsgBuf;
				FormatMessage( FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS,
								NULL, le, 0, (LPTSTR) &lpMsgBuf, 0, NULL );
				string msg = "Generate GR Error: Unable to create directory\n";
				msg += configDir + "\n";
				msg += (LPCSTR)lpMsgBuf;
				msg += ".";
				throw umt_exception(msg);
			}
		}

		Udm::SmartDataNetwork config(GReATConfig::diagram);
		config.CreateNew(name,"GReATConfig", GReATConfig::RootFolder::meta, Udm::CHANGES_LOST_DEFAULT);
		GReATConfig::RootFolder crf = GReATConfig::RootFolder::Cast(config.GetRootObject());
				
		// transformation
		Packets_t confPacket;
		Packets_t umtPacket;

		confPacket.push_back(crf);
		umtPacket.push_back(cnf);

		UMT2Config_0 uMT2Config_0;
		uMT2Config_0( confPacket, umtPacket);

//		std::vector< GReATConfig::TemporaryFileUsage> tempFileUsages;
//		GReATConfig::UsageInformation ui= crf.UsageInformation_child();
//		GReATConfigUsage gcu( ui);
//		gcu.getTemporaryFileUsages( tempFileUsages);
		try {
			GReATConfig::UsageInformation ui= crf.UsageInformation_child();
			TempDNProcessor tdnp( getUdmProjectFolder( cnf));
			tdnp( rf, ui);
		}
		catch (TempDNEx& ex){
			AfxMessageBox( ex.what().c_str(), MB_ICONSTOP);
			config.CloseWithUpdate();
			return;
		}

		config.CloseWithUpdate();
		CString msg = "Files Generated\n";
		msg = msg + name.c_str() + "\n";
		msg = msg + configDir.c_str() + trans.c_str() + "\n";
		msg = msg + configDir.c_str() + udmprj.c_str() + "\n";
		
		AfxMessageBox(msg ,MB_ICONINFORMATION);
		//////////////////////////////////////////////////////////////////////////////
	}
	/*catch(udm_exception &e)
	{
		AfxMessageBox(e.what());
	} 
	Exception is caught in ComponentDll.cpp
	*/										
}



#ifdef _DEBUG
/*****************************************************/
/* Debug time helper function. If the object has an  */
/* attribute called "name", this function retreives  */
/* it to help you to find it in the model during the */
/* application development.	Usualy every GME Object	 */
/* has a "name" attribute. If an object hapens not	 */
/* to have it,function retreives <no name specified>.*/
/*****************************************************/
string CUdmApp::ExtractName(Udm::Object ob)
{
	Uml::Class cls= ob.type();				
	set<Uml::Attribute> attrs=cls.attributes();		
	
	// Adding parent attributes
	set<Uml::Attribute> aattrs=Uml::AncestorAttributes(cls);
	attrs.insert(aattrs.begin(),aattrs.end());

	for(set<Uml::Attribute>::iterator ai = attrs.begin();ai != attrs.end(); ai++) 
	{
		if(string(ai->type())=="String")
		{
			string str=ai->name();
			if(str=="name")
			{
				string value=ob.getStringAttr(*ai);
				if(value.empty())value="<empty string>";
				return value;
			}			
		}				
	}	
	return string("<no name specified>");
}
#endif
