///////////////////////////////////////////////////////////////////////////
// RawComponent.cpp, the main RAW COM component implementation file
// This is the file (along with its header RawComponent.h)
// that the component implementor is expected to modify in the first place
//
///////////////////////////////////////////////////////////////////////////
#include <iostream.h>
#include <fstream.h>

#include "stdafx.h"

#include "ComHelp.h"
#include "GMECOM.h"
#include <ComponentConfig.h>
#include "RawComponent.h"


CString progName;

// this method is called after all the generic initialization is done
// this should be empty, unless application-specific initialization is needed
STDMETHODIMP RawComponent::Initialize(struct IMgaProject *) {
	return S_OK;
}

// this is the obsolete component interface
// this present implementation either tries to call InvokeEx, or returns an error;
STDMETHODIMP RawComponent::Invoke(IMgaProject* gme, IMgaFCOs *models, long param) {
#ifdef SUPPORT_OLD_INVOKE
	CComPtr<IMgaFCO> focus;
	CComVariant parval = param;
	return InvokeEx(gme, focus, selected, parvar);
#else
	if(interactive) {
		AfxMessageBox("This component does not support the obsolete invoke mechanism");
	}
	return E_MGA_NOT_SUPPORTED;
#endif
}


// This is the main component method for interpereters and plugins. 
// May als be used in case of invokeable addons
STDMETHODIMP RawComponent::InvokeEx( IMgaProject *project,  IMgaFCO *currentobj,  
									IMgaFCOs *selectedobjs,  long param) {
	COMTRY {
	  if(interactive) {
		CComBSTR projname;
		CComBSTR focusname = "<nothing>";
		CComPtr<IMgaTerritory> terr;
		COMTHROW(project->CreateTerritory(NULL, &terr));
		COMTHROW(project->BeginTransaction(terr));
		try {
			COMTHROW(project->get_Name(&projname));
			if(currentobj) COMTHROW(currentobj->get_Name(&focusname));
			AfxMessageBox("RAW Com Component --- Plugin!!!! Sample (project: " + CString(projname) +
						", focus: " + CString(focusname));
			COMTHROW(project->CommitTransaction());
		}	catch(...) { project->AbortTransaction(); throw; }
		
	  } 
	} COMCATCH(;);
}

// GME currently does not use this function
// you only need to implement it if other invokation mechanisms are used
STDMETHODIMP RawComponent::ObjectsInvokeEx( IMgaProject *project,  IMgaObject *currentobj,  IMgaObjects *selectedobjs,  long param) {
	if(interactive) {
		AfxMessageBox("Tho ObjectsInvoke method is not implemented");
	}
	return E_MGA_NOT_SUPPORTED;
}


// implement application specific parameter-mechanism in these functions:
STDMETHODIMP RawComponent::get_ComponentParameter(BSTR name, VARIANT *pVal) {
	return S_OK;
}

STDMETHODIMP RawComponent::put_ComponentParameter(BSTR name, VARIANT newVal) {
	return S_OK;
}


#ifdef GME_ADDON

// these two functions are the main 
STDMETHODIMP RawComponent::GlobalEvent(globalevent_enum event) { 
	if(event == GLOBALEVENT_UNDO) 
	{	//AfxMessageBox("UNDO!!");
	}
	return S_OK; 
}

STDMETHODIMP RawComponent::ObjectEvent(IMgaObject * obj, unsigned long eventmask, VARIANT v) 
{
	if(eventmask & OBJEVENT_CREATED) 
	{	CComBSTR objID;
		COMTHROW(obj->get_ID(&objID));
		//AfxMessageBox( "Object created! ObjID: " + CString(objID));
		objtype_enum type;
		COMVERIFY(obj->get_ObjType(&type));
		if(type == OBJTYPE_CONNECTION) 
		{	CComQIPtr<IMgaConnection> ciConnection(obj);
			CString skname = GetConnectionKindName(ciConnection);
			if(skname == "PatternComposition" || skname == "PatternAssociation")
			{	// Get the objects from references and see if this association can exist
				SetAttribute(CComQIPtr<IMgaSimpleConnection>(ciConnection));
			}
			else if(skname == "Binding" )
			{	ciConnection->put_RegistryValue((CBstrIn)"color", (CBstrIn)"0xAA00AA");
			}
			else if(skname == "AssociationClass" )
			{	CString src, dst;
				if(!GetSrcDstRolenames(CComQIPtr<IMgaSimpleConnection>(ciConnection)))
					return S_OK;
			}

		}
	}
	else if(eventmask & OBJEVENT_CONNECTED)
	{	CComBSTR objID;
		COMTHROW(obj->get_ID(&objID));
		//	AfxMessageBox( "Object created! ObjID: " + CString(objID));
		objtype_enum type;
		COMVERIFY(obj->get_ObjType(&type));
		if(type == OBJTYPE_ATOM) 
		{	CComQIPtr<IMgaAtom> ciAtom(obj);
			CBstr AtomName;
			COMVERIFY(ciAtom->get_Name(AtomName));
			//	AfxMessageBox(CString(AtomName));
		}
	}
	else if(eventmask & OBJEVENT_ATTR)
	{	CComBSTR objID;
		COMTHROW(obj->get_ID(&objID));
		//AfxMessageBox( "Object created! ObjID: " + CString(objID));
		objtype_enum type;
		COMVERIFY(obj->get_ObjType(&type));
		if(type == OBJTYPE_CONNECTION) 
		{	CComQIPtr<IMgaSimpleConnection> ciConnection(obj);
			CBstr bstr;
			{	CComPtr<IMgaMetaFCO> meta;
				COMVERIFY(ciConnection->get_Meta(&meta));
				COMVERIFY(meta->get_Name(bstr));
			}
			CString skname(bstr);
			CBstr connAction;

			if(skname == "PatternComposition" || skname == "PatternAssociation")
			{	if ( ! SUCCEEDED( ciConnection->get_StrAttrByName((CBstrIn)"Action", connAction) ) )
					return S_OK;
				if(connAction == "CreateNew")
				{	ciConnection->put_RegistryValue((CBstrIn)"color", (CBstrIn)"0x0000ff");
				}
				else if (connAction == "Delete")
				{	ciConnection->put_RegistryValue((CBstrIn)"color", (CBstrIn)"0xff0000");
				}
				else
				{	ciConnection->put_RegistryValue((CBstrIn)"color", (CBstrIn)"0x000000");
				}
			}
			if(skname == "PatternComposition")
			{	CBstr srcAction;
				
				CComPtr<IMgaFCO> iChildObject;
				COMVERIFY( ciConnection->get_Src(&iChildObject));
				if ( ! SUCCEEDED( iChildObject->get_StrAttrByName((CBstrIn)"Action", srcAction) ) )
					return S_OK;
				if(CString(connAction) != CString(srcAction))
					iChildObject->put_StrAttrByName((CBstrIn)"Action", (CBstrIn)connAction);
				return S_OK;
			}
		}
		else if(type == OBJTYPE_REFERENCE) 
		{	CComQIPtr<IMgaReference> ciRef(obj);
			CBstr bstr;
			{	CComPtr<IMgaMetaFCO> meta;
				COMVERIFY(ciRef->get_Meta(&meta));
				COMVERIFY(meta->get_Name(bstr));
			}
			CString skname(bstr);
			
			if(skname == "PatternClass")
			{	CBstr connAction;
				if ( ! SUCCEEDED( ciRef->get_StrAttrByName((CBstrIn)"Action", connAction) ) )
					return S_OK;
				if(connAction == "CreateNew" || connAction == "Delete")
				{	CComPtr<IMgaConnPoints> cpts;
					COMVERIFY( ciRef->get_PartOfConns(&cpts));
					
					MGACOLL_ITERATE(IMgaConnPoint, cpts) 
					{	CComQIPtr<IMgaConnection> ciConnection;
						COMVERIFY(MGACOLL_ITER->get_Owner(&ciConnection));
						CString skname = GetConnectionKindName(ciConnection);
						if(skname == "PatternComposition" || skname == "PatternAssociation")
						{	SetAttribute(CComQIPtr<IMgaSimpleConnection>(ciConnection));
						}
						
					} MGACOLL_ITERATE_END;
				}
				
			}
		}
		/*else if(type == OBJTYPE_ATOM)
		{	CComQIPtr<IMgaAtom> ciAtom(obj);
			CBstr bstr;
			{	CComPtr<IMgaMetaFCO> meta;
				COMVERIFY(ciAtom->get_Meta(&meta));
				COMVERIFY(meta->get_Name(bstr));
			}
			CString skname(bstr);
			if(skname == "AttributeMapping")
			{	CBstr amStr;
				if ( ! SUCCEEDED( ciAtom->get_StrAttrByName((CBstrIn)"ExpressionString", amStr) ) )
					return S_OK;

				CString filename= getenv("GREAT_PATH");
				filename += "\\tmp\\zz_amtp.cpp";
				CString amexp(amStr);

				std::ofstream amout((LPCSTR)filename);
				amout << (LPCSTR)amexp;
				amout.close();
				
				progName = "\"C:\\Program Files\\Microsoft Visual Studio\\Common\\MSDev98\\Bin\\msdev.exe\"";
				CString systemcall = "call " + progName + " " + "\"" + filename + "\"";

				system((LPCSTR)systemcall);
				
				std::ifstream amin((LPCSTR)filename);
				
				
				CString aout;
				char ch[2];
				ch[1] = '\0';

				while(amin.get(ch[0]))	{	aout += ch; }
				amin.close();
					
				if(amexp == aout)
					return S_OK;

				if( !SUCCEEDED( ciAtom->put_StrAttrByName((CBstrIn)"ExpressionString", (CBstrIn)aout) ) )
					return S_OK;
			
			}

		}*/
	}
	
	return S_OK;
}

int RawComponent::SetAttribute(CComQIPtr<IMgaSimpleConnection> ciConnection)
{	CComPtr<IMgaFCO> isrc;
	CComPtr<IMgaFCO> idst;
	
	COMVERIFY(ciConnection->get_Src(&isrc) );
	CComQIPtr<IMgaReference> isrcAtom(isrc);

	COMVERIFY(ciConnection->get_Dst(&idst) );
	CComQIPtr<IMgaReference> idstAtom(idst);
					
	CBstr srcAction;
	if ( ! SUCCEEDED( isrcAtom->get_StrAttrByName((CBstrIn)"Action", srcAction) ) )
		return S_OK;
	CBstr dstAction;
	if ( ! SUCCEEDED( idstAtom->get_StrAttrByName((CBstrIn)"Action", dstAction) ) )
		return S_OK;
	CBstr connAction;
	if ( ! SUCCEEDED( ciConnection->get_StrAttrByName((CBstrIn)"Action", connAction) ) )
		return S_OK;
	
	if(srcAction == "CreateNew" && connAction != "CreateNew" || dstAction == "CreateNew" && connAction != "CreateNew")
	{	if( !SUCCEEDED( ciConnection->put_StrAttrByName((CBstrIn)"Action", (CBstrIn)"CreateNew") ) )
			return S_OK;
	}
	else if(srcAction == "Delete" && connAction != "Delete"|| dstAction == "Delete" && connAction != "Detele")
	{	if( !SUCCEEDED( ciConnection->put_StrAttrByName((CBstrIn)"Action", (CBstrIn)"Delete") ) )
			return S_OK;
	}
	
	return S_OK;
}

CString RawComponent::GetConnectionKindName(CComQIPtr<IMgaConnection> ciConn)
{	CBstr bstr;
	{	CComPtr<IMgaMetaFCO> meta;
		COMVERIFY(ciConn->get_Meta(&meta));
		COMVERIFY(meta->get_Name(bstr));
	}
	return CString(bstr);
}

bool RawComponent::GetSrcDstRolenames(CComQIPtr<IMgaSimpleConnection> ciConn)
{
	CString name;
	CBstr srcRolename, dstRolename;

	CComPtr<IMgaModel> parent;
	ciConn->get_ParentModel(&parent);
	
	CComPtr<IMgaFCO> iObject,iNObject,iRealConn;
	COMVERIFY( ciConn->get_Dst(&iObject));
	COMVERIFY( ciConn->get_Src(&iRealConn));

	CComQIPtr<IMgaConnection> srcRoleConn, dstRoleConn;
	{	CComPtr<IMgaConnPoints> opts;
		COMVERIFY( iRealConn->get_PartOfConns(&opts));
						
		MGACOLL_ITERATE(IMgaConnPoint, opts) 
		{	CComQIPtr<IMgaConnection> cioConnection;
			COMVERIFY(MGACOLL_ITER->get_Owner(&cioConnection));
			CString sokname = GetConnectionKindName(cioConnection);
			if(sokname == "Src")
			{	srcRoleConn = cioConnection;
			}
			else if(sokname == "Dst")
			{	dstRoleConn = cioConnection;
			}
			
		} MGACOLL_ITERATE_END;
	}
		
	objtype_enum type;
	COMVERIFY(iObject->get_ObjType(&type));
	while(type == OBJTYPE_REFERENCE) 
	{	CComQIPtr<IMgaReference> iRef = iObject;
		
		CBstr refName;
		COMVERIFY(iRef->get_Name(refName));
		CString rName(refName);

		COMVERIFY(iRef->get_Referred(&iNObject));
		iObject = iNObject;
		// VIZA fix null ref association class bug
		if ( iObject) {
			COMVERIFY(iObject->get_ObjType(&type));
		} else {
			break;
		}
	}
	if ( iObject) {
	
		CComQIPtr<IMgaAtom> iClass(iObject);
		
		// For Debug Purpose
		{	CBstr bstr;
			COMVERIFY(iClass->get_Name(bstr));
			name= bstr;
		}

		CComPtr<IMgaFCOs> iClassRefs;
		iClass->get_ReferencedBy(&iClassRefs);

		iClassRefs->Append(iClass);

		MGACOLL_ITERATE(IMgaFCO, iClassRefs)
		{	
			CComPtr<IMgaFCO> iClassObj(MGACOLL_ITER);

			CComPtr<IMgaMetaFCO> meta;
			CBstr cobsName;
			COMVERIFY(iClassObj->get_Meta(&meta));
			COMVERIFY(meta->get_Name(cobsName));
			CString cobjName(cobsName);
			if(cobsName != "ClassCopy" && cobjName != "Class")
				continue;
			
			CComPtr<IMgaConnPoints> cpts;
			COMVERIFY( iClassObj->get_PartOfConns(&cpts));
							
			MGACOLL_ITERATE(IMgaConnPoint, cpts) 
			{	CComQIPtr<IMgaConnection> ciConnts;
				COMVERIFY(MGACOLL_ITER->get_Owner(&ciConnts));
				CString skname = GetConnectionKindName(ciConnts);
				if(skname == "AssociationClass")
				{	CComPtr<IMgaFCO> iSrcFco;
					CComQIPtr<IMgaSimpleConnection> ciConnection(ciConnts);
					COMVERIFY( ciConnection->get_Src(&iSrcFco));
					CComQIPtr<IMgaAtom> iConnector(iSrcFco);
					
					CComPtr<IMgaConnPoints> opts;
					COMVERIFY( iConnector->get_PartOfConns(&opts));
									
					MGACOLL_ITERATE(IMgaConnPoint, opts) 
					{	CComQIPtr<IMgaConnection> cioConnection;
						COMVERIFY(MGACOLL_ITER->get_Owner(&cioConnection));
						CString sokname = GetConnectionKindName(cioConnection);
						if(sokname == "Src")
						{	cioConnection->get_StrAttrByName((CBstrIn)"srcRolename", srcRolename); 
						}
						else if(sokname == "Dst")
						{	cioConnection->get_StrAttrByName((CBstrIn)"dstRolename", dstRolename);
						}
					} MGACOLL_ITERATE_END;
				}
			} MGACOLL_ITERATE_END;
		} MGACOLL_ITERATE_END;
	}// if ( iObject)
	
	int stophere= 0;
	if(srcRoleConn)	{
		if( srcRolename == "") {
			stophere++;
		} else {
			srcRoleConn->put_StrAttrByName((CBstrIn)"srcRolename", (CBstrIn) srcRolename);
		}
	}
	if(dstRoleConn) {
		if ( dstRolename == "") {
			stophere--;
		} else {
		dstRoleConn->put_StrAttrByName((CBstrIn)"dstRolename", (CBstrIn) dstRolename);
		}
	}
	return true;
}


#endif
